#partoffer.py
from pymomo.utilities.formatting import indent_block

def convert(arg, **kwargs):
	if isinstance(arg, PartOffer):
		return arg

	if not isinstance(arg, dict):
		raise ValueError("conversion to part_offer from anything other than dict not supported")

	req = ['seller', 'moq', 'packaging', 'breaks', 'stock_quantity']
	for r in req:
		if r not in arg:
			raise ValueError('invalid dictionary used to initialize part_offer')

	return PartOffer(arg['seller'], arg['packaging'], arg['breaks'], arg['stock_quantity'], arg['moq'])

def default_formatter(arg, **kwargs):
	return str(arg)

class PartOffer:
	def __init__(self, seller, packaging, breaks, stock, moq):
		self.seller = seller
		self.packaging = packaging
		self.invalid = False

		self.breaks = breaks
		self.stock_quantity = stock
		
		if moq is not None:
			self.moq = moq
		elif len(self.breaks.breaks) > 0:
			self.moq = self.breaks.breaks[0][0]
		else:
			self.moq = None

	def best_price(self, quantity):
		if self.invalid:
			return None

		return self.breaks.unit_price(quantity)

	def __str__(self):
		res = "Seller: %s\n  Packaging: %s\n  Stock: %d\n  Minimum Order: %s" % (self.seller, self.packaging, self.stock_quantity, str(self.moq))
		breakstr = str(self.breaks)
		if breakstr != "":
			res += "\n  Breaks\n"
			res += indent_block(breakstr, 4)

		return res



#dump_fat.py

import sys
import struct

def read_sector(dev, s, num=1):
	dev.seek(s*512)
	return dev.read(512*num)

def readNum(buf, f, offset):
	fmt = '<' + f

	(res,) = struct.unpack_from(fmt, buf, offset=offset)

	return res

def read16(buf, offset):
	return readNum(buf, 'H', offset)

def read32(buf, offset):
	return readNum(buf, 'I', offset)

def read8(buf, offset):
	return readNum(buf, 'B', offset)

def read_fat(dev, sect):
	fmt = '<128I'

	return struct.unpack(fmt, read_sector(dev, sect))

def read_mbr(dev):
	part_fmt = "<B3xB3xII"
	magic_fmt = "<H"
	mbr = read_sector(dev, 0)

	(magic,) = struct.unpack_from(magic_fmt, mbr, offset=510)

	print magic

	if magic != 0xAA55:
		raise ValueError("Invalid MBR Magic Number: 0x%X" % magic)

	parts = [struct.unpack_from(part_fmt, mbr, offset=446+16*x) for x in xrange(0,4)]

	pdicts = []
	for part in parts:
		d = {"first_sector": part[2], "num_sectors": part[3]}
		pdicts.append(d)

	return pdicts

def get_clusterval(dev, cluster, part):
	fat_begin = part['part_start'] + part['num_reserved']

	sect_idx = cluster / (128) + fat_begin
	offset = cluster % (128)

	clusters = read_fat(dev, sect_idx)

	return clusters[offset]

def get_chain(dev, start_cluster):
	fat 

def read_fat32part(dev, offset):
	part = read_sector(dev, offset)

	partinfo = {}

	partinfo['bytes_per_sector'] = read16(part, 0x0B)
	partinfo['sects_per_cluster'] = read8(part, 0x0D)
	partinfo['num_reserved'] 	= read16(part, 0x0E)
	partinfo['num_fats'] = read8(part, 0x10)
	partinfo['sects_per_fat'] = read32(part, 0x24)
	partinfo['root_cluster'] = read32(part, 0x2C)
	partinfo['part_start'] = offset
	partinfo['cluster_start'] = offset + partinfo['num_reserved'] + partinfo['num_fats']*partinfo['sects_per_fat']

	return partinfo	

def read_cluster(dev, cluster, part):
	start_sect = (cluster - 2)*part['sects_per_cluster'] + part['cluster_start']

	return read_sector(dev, start_sect, part['sects_per_cluster'])

def read_dir(dev, cluster, part):
	data = read_cluster(dev, cluster, part)

	dirfmt = '<11sB8xH4xHI'

	numents = len(data) / 32

	ents = []
	for i in xrange(0,numents):
		(name, attrib, clust_high, clust_low, size) = struct.unpack_from(dirfmt, data, offset=32*i)
		if name[0] == '\0':
			continue
		if name[0] == '\xE5':
			continue
		if attrib & 0xF == 0xF:
			continue

		clust = clust_high << 16 | clust_low
		ent = {'name': name, 'cluster': clust, 'size': size, 'attrib': attrib}

		ents.append(ent)

	return ents

def list_cluster_chain(dev, part, start, num=20):
	cluster = start

	for i in xrange(0, num):
		n = get_clusterval(dev, cluster, part)
		print "%d: Cluster 0x%X: 0x%X" % (i+1, cluster, n)

		cluster = n

if len(sys.argv) != 2:
	print "Usage: dump_fat <device>"
	sys.exit(1)

with open(sys.argv[1], "rb", buffering=-1) as dev:
	parts = read_mbr(dev)
	part = parts[0]

	print "Partition 1"
	print "* Start: %d" % part['first_sector']
	print "* Size:  %d sectors" % part['num_sectors']

	f32info = read_fat32part(dev, part['first_sector'])

	ents = read_dir(dev, f32info['root_cluster'], f32info)
	log = ents[0]

	print "Listing chain for file: %s" % log["name"]
	list_cluster_chain(dev, f32info, log["cluster"], num=13)


#instruction.py

from instructions import *

class Instruction:
	"""
	A representation of a pic 8-bit enhanced midrange instruction
	built from either a string mnemonic or decoded from a binary
	opcode.
	"""

	Mnemonics = {}

	def __init__(self, instruction):
		"""
		Given either a mnemonic string or a binary opcode,
		build the corresponding instruction object.
		"""

		if isinstance(instruction, basestring):
			self._parse_mnemonic(instruction)
		else:
			self._parse_opcode(instruction)

	def _parse_mnemonic(self, instruction):
		instr = instruction.lstrip().rstrip()
		op,sep,args = instr.partition(' ')

		if len(args) != 0: 
			arglist = map(lambda x: x.lstrip().rstrip(), args.split(","))
		else:
			arglist = []

		if op not in Instruction.Mnemonics:
			raise ValueError("Unknown Mnemonic passed to _parse_mnenomic: %s" % op)
		
		self.op = Instruction.Mnemonics[op](arglist)

	def encode(self):
		"""
		Get the binary opcode for this instruction
		"""

		return self.op.encode()


Instruction.Mnemonics['retlw'] = RetlwInstruction
Instruction.Mnemonics['goto'] = GotoInstruction


#bom_matcher.py
#A generic interface for services that take in identifying information about a
#part and return price offerings for that part.

from pymomo.utilities.typedargs import *
from pymomo.exceptions import *
from partcache import PartCache
import uuid
from decimal import Decimal

class BOMMatcher(object):
	def __init__(self):
		self.requests = []
		self.matched = {}
		self.errors = {}
		self.cache = PartCache()

		self.ref_index = {}

	def _canonicalize_name(self, name):
		"""
		Given a name, like a distributor's name, convert it to
		lowercase and remove all spaces,_ or - characters 
		"""

		return name.lower().replace(' ', '').replace('-', '').replace('_', '')

	def _build_reference(self, part):
		if "distpn" in part:
			return "__SKU__" + self._canonicalize_name(str(part['distpn'])) + self._canonicalize_name(str(part['dist']))

		else:
			mpn = part['mpn']
			manu = part['manu']

			return "__MPN__" + self._canonicalize_name(str(part['mpn'])) + self._canonicalize_name(str(part['manu']))

	@param("mpn", "string", "not_empty", desc="manufacturer's part number")
	@param("manu", "string", desc="manufacturer")
	@param("ref", "string", desc="reference number")
	def add_by_mpn(self, mpn, manu=None, ref=""):
		"""
		Find a part by its manufacturer's part number

		Optionally filter to make sure that the manufacturer is equal to the specified
		'manu' string in case the mpn is not globally unique.  Comparison between
		manufacturer strings, if given, is done in a case insensitive manner after removing
		all - and _ characters from both strings, so e.g. Digi-Key matches digikey.
		
		If ref is given, the returned information will contain the reference number so that
		it can be unambiguously matched with the source that generated the request for pricing
		"""

		part = {'mpn': mpn, 'manu': manu, 'external_ref': ref}
		part['ref'] = self._build_reference(part)

		self.requests.append(part)
		if ref != "":
			self._add_reference(part['ref'], ref)

	@param("distpn", "string", "not_empty", desc="distributor part number")
	@param("dist", "string", desc="distributor name")
	@param("ref", "string", desc="reference identifier")
	def add_by_distpn(self, distpn, dist="", ref=""):
		"""
		Find a part by its distributor part number

		Optionally filter to make sure that the distributor is equal to the specified
		'dist' string in case the distpn is not globally unique.  Comparison between
		distributor strings, if given, is done in a case insensitive manner after removing
		all - and _ characters from both strings, so e.g. Digi-Key matches digikey.

		If ref is given, the returned information will contain the reference number so that
		it can be unambiguously matched with the source that generated the request for pricing
		"""

		part = {'distpn': distpn, 'dist': dist, 'external_ref': ref}
		part['ref'] = self._build_reference(part)

		self.requests.append(part)
		if ref != "":
			self._add_reference(part['ref'], ref)

	@param("part", "logical_part", "matchable", desc="component to look up")
	def add_part(self, part):
		"""
		Find a logical part based on its embedded metadata
		"""

		if part.manu is not None and part.mpn is not None:
			self.add_by_mpn(part.mpn, part.manu, ref=part.name)

			#If we have information about both a distributor and a mpn
			#save both bits of information so that we can filter in case the MPN
			#is not uniquely identifying
			if part.dist is not None:
				self.requests[-1]['require_dist'] = part.dist
			if part.distpn is not None:
				self.requests[-1]['require_distpn'] = part.distpn
		else:
			self.add_by_distpn(part.distpn, part.dist, ref=part.name)

	def _add_reference(self, internal, external):
		if external in self.ref_index:
			raise ArgumentError("attempted to add same reference number twice", reference=external, old=self.ref_index, new=internal)
		
		self.ref_index[external] = internal
	
	@annotated
	def clear(self):
		"""
		Clear all pending requests and results
		"""

		self.requests = []
		self.matched = {}
		self.errors = {}
		self.ref_index = {}

	@annotated
	def match_all(self):
		"""
		Attempt to match all parts.

		Parts that are present in the part cache already are not
		passed on to the matching engine but instead returned directly
		from the cache.  Parts that are uniquely matched are cached 
		for future reference.
		"""

		#Figure out which items are cached
		cache_status = map(lambda x: self.cache.try_get(x['ref']), self.requests)
		uncached = [x[1] for x in enumerate(self.requests) if cache_status[x[0]] is None]

		reqs = [x for x in self.requests]
		self.requests = uncached
		self._match()

		#Perform additional checks to make sure that the match engines are returning
		#just the results that we want and not extras.
		for req in reqs:
			if req['ref'] in self.matched:
				parts = self.matched[req['ref']]

				#Some match engines only match the MPN, not the manufacturer's name, so enforce that here
				if 'manu' in req and req['manu'] is not None:
					self.matched[req['ref']] = filter(lambda x: self._canonicalize_name(req['manu']) == self._canonicalize_name(x.manu), parts)

				#Make sure that if we specified a specific distributor that this part is carried by that distributor
				dist = None
				if 'require_dist' in req:
					dist = req['require_dist']
				if 'dist' in req and req['dist'] is not None:
					dist = req['dist']
				if dist is not None:
					self.matched[req['ref']] = filter(lambda x: self._dist_in_offers(x, dist), parts)

		#Update the cache with the new results
		for key, value in self.matched.iteritems():
			if len(value) == 1:
				self.cache.set(key, value[0])

		#Add in all of the cached results
		for i, x in enumerate(cache_status):
			if x is not None:
				self.matched[reqs[i]['ref']] = [x]

		#Reset the list of requests to include cached and uncached values
		self.requests = reqs

	@return_type("map(string, integer)")
	def match_summary(self):
		"""
		Show which parts have been added, matched or had errors
		"""

		total_reqs = len(self.requests)
		total_matched = len(self.matched)
		total_errors = len(self.errors)

		multimatch = filter(lambda x: len(x) > 1, self.matched.itervalues())
		total_multi = len(multimatch)
		
		ret =  {'Requested Parts': total_reqs, 'Matched Parts': total_matched, 
				'Parts with Errors': total_errors, 'Multiple Matches': total_multi}

		return ret

	@return_type("string")
	def all_matched(self):
		"""
		Check if all requested parts have been matched to exactly 1 physical part
		"""

		if len(self.matched) == len(self.requests):
			multimatch = filter(lambda x: len(x) > 1, self.matched.itervalues())
			total_multi = len(multimatch)

			if total_multi == 0:
				return True

		return False

	@param("reference", "string", "not_empty", desc="reference number")
	@return_type("string")
	def is_matched(self, reference):
		"""
		Check if reference number is matched to a physical part
		"""

		if reference not in self.ref_index:
			return False

		intref = self.ref_index[reference]
		if intref in self.matched:
			return True

		return False

	@return_type("map(string, string)")
	def match_details(self):
		"""
		List the matching status of each part according to its reference number
		"""

		stats = {}

		for r in self.requests:
			refnum = r['ref']
			if 'external_ref' in r and r['external_ref'] != '':
				refnum = r['external_ref']

			if r['ref'] in self.matched:
				if len(self.matched[r['ref']]) == 1:
					stats[refnum] = 'Matched to a unique item'
				else:
					stats[refnum] = '%d separate matches' % (len(self.matched[r['ref']]),)

			elif r['ref'] in self.errors:
				stats[refnum] = self.errors[r['ref']]['msg']
			else:
				stats[refnum] = 'Match has not been attempted'

		return stats

	@param("reference", "string", "not_empty", desc="user reference number")
	@return_type("list(physical_part)")
	def match_info(self, reference):
		"""
		Return matched part info by reference number

		If the part matched multiple items, they are all returned.
		"""

		if reference not in self.ref_index:
			raise ArgumentError("unknown reference number passed", reference=reference)

		intref = self.ref_index[reference]

		if intref not in self.matched:
			raise ArgumentError("part was not matched", reference=reference, internal_reference=intref)

		info = self.matched[intref]

		return info

	@param("quantity", "integer", "positive", desc="requested number of parts")
	@param("reference", "string", "not_empty", desc="reference number")
	@param("seller", "string", desc="required seller of the part")
	@return_type("string")
	def price(self, reference, quantity, seller):
		"""
		Return the best price for N parts from seller as a Decimal
		"""

		info = self.match_info(reference)
		can_seller = self._canonicalize_name(seller)

		if len(info) > 1:
			raise ArgumentError("part is not uniquely matched", reference=reference)

		offers = filter(lambda x: self._canonicalize_name(x.seller)==can_seller, info[0].offers)

		best = Decimal.from_float(1e15)
		for offer in offers:
			price = offer.best_price(quantity)
			if price is not None and price < best:
				best = price

		return best

	def price_advanced(self, reference, quantity, requirements):
		"""
		Find the price of a single item subject to various requirements

		requirements must have a validate function that takes two arguments: an offer and the quantity		
		"""

		info = self.match_info(reference)

		if len(info) > 1:
			return {'error': 'Part not uniquely matched'}

		offers = filter(lambda x: requirements.validate(x, quantity), info[0].offers)

		best = Decimal.from_float(1e15)
		best_offer = None
		for offer in offers:
			price = offer.best_price(quantity)
			if price is not None and price < best:
				best = price
				best_offer = offer

		if best_offer is None:
			return {'error': 'No valid offers found'}
		else:
			return {'price': best, 'offer': offer}

	def prices(self, quantities, requirements):
		"""
		Return the prices for specified parts at specified quantities.

		requirements must have a validate function that takes two arguments: an offer and the quantity
		quantities must be a dictionary mapping reference numbers to quantities 
		"""

		result = {}

		for ref,quantity in self.quantities.iteritems():
			info = self.match_info(ref)

			if len(info) > 1:
				result[ref] = {'error': 'Part not uniquely matched'}
				continue

			offers = filter(lambda x: requirements.validate(x, quantity), info[0].offers)

			best = Decimal.from_float(1e15)
			best_offer = None
			for offer in offers:
				price = offer.best_price(quantity)
				if price is not None and price < best:
					best = price
					best_offer = offer

			if best_offer is None:
				result[ref] = {'error', 'No valid offers found'}
			else:
				result[ref] = {'price': best, 'offer': offer}

		return result

	def _dist_in_offers(self, part, dist):
		"""
		Given a physical_part, make sure it contains an offer from distributor dist
		"""

		dist = self._canonicalize_name(dist)

		for offer in part.offers:
			if self._canonicalize_name(offer.seller) == dist:
				return True

		return False



#hexfile.py

import intelhex
from pymomo.utilities.typedargs.types import *
from pymomo.exceptions import *
from pymomo.utilities.typedargs.annotate import *

def print_ranges(ranges):
	print "Defined Address Ranges"
	for start,stop in ranges:
		print "[0x%X, 0x%X]" % (start, stop) 

def print_errors(errors):
	print "Comparing hex files"

	if len(errors) == 0:
		print "Files are equivalent"
		return

	for error in errors:
		src = 'first'
		this = error['this']
		other = error['other']
		if error['direction'] == 'reverse':
			src = 'second'
			this = error['other']
			other = error['this']

		print "Mismatch in range (0x%X, 0x%X) defined in %s file, file 1: 0x%X, file 2: 0x%X @ address 0x%x" % (error['range'][0], error['range'][1], src, this, other, error['address'])

@context("HexFile")
class HexFile(object):
	"""
	A generalized container object for representing sparse memory views like those
	contained in intel HEX files.  It uses the intelhex format for reading hex files
	but contains the ability to modify the read information to acount for symantic
	details like word size and default padding.  It's primary purpose is to support
	pic12 and pic24 hex files which have 14-bit and 24-bit words respectively.
	"""

	@param("source", "path", "readable", desc="hex file to load")
	@param("wordsize", "integer", "positive", desc="native word size of architecture in bits")
	@param("filewords", "integer", "positive", desc="bytes per word in hex file")
	@param("skip", "integer", "positive", desc="valid addresses: 1=all, 2=even")
	@param("cutoff", "integer", "positive", desc="ignore addresses above this value")
	def __init__(self, source, wordsize, filewords, skip=1, cutoff=0xFFFF):
		"""
		Create a HexFile object representing the source file 
		"""

		ih = intelhex.IntelHex(source)

		self.wordsize = wordsize
		self.filewords = filewords
		self.skip = skip
		self.fill = (1 << (self.wordsize)) - 1
		self.cutoff = cutoff
		self._buf = {}
		self._convert_from_hex(ih)

	def _read_word(self, ih, addr):
		"""
		Read filewords bytes from ih starting at addr.  Raise an error if some of
		the bytes are not implemented. 
		"""

		word = 0

		for i in xrange(0, self.filewords):
			a = addr+i
			if a not in ih._buf:
				print a
				raise InternalError("Hex file had incorrect word size at address %d" % addr)

			val = ih._buf[a]
			word |= (val & 0xFF) << (i*8)

		mask = (1 << self.wordsize) -1
		if word > mask:
			raise InternalError("Hex file had word exceeding the specified architecture word width (%d bits), value was %d @ address 0x%X" % (self.wordsize, word, addr))

		return word

	def _convert_from_hex(self, ih):
		"""
		Convert from a padded hex file to a native word representation
		"""

		start = ih.minaddr()
		end = ih.maxaddr()

		if start % self.filewords != 0:
			raise InternalError("Hex file did not start on a multiple of its file word size. (base address was %d, word size was %d" % (start, self.filewords))

		aligned_start = start / self.filewords

		for i in xrange(start, end, self.filewords):
			if i not in ih._buf or i>self.cutoff:
				continue

			addr = 	i/self.filewords * self.skip
			word = self._read_word(ih, i)
			self._buf[addr] = word

	def _convert_to_hex(self):
		"""
		Convert from a native word representation to a padded hex file
		using the parameters specified upon creation of this HexFile object
		"""

		ih = intelhex.IntelHex()

		addrs = self.addresses()
		mask = (1 << self.wordsize) -1

		for addr in addrs:
			start = (addr/self.skip) * self.filewords

			val = self._buf[addr] & mask

			for i in xrange(0, self.filewords):
				byte_i = (val >> (8*i)) & 0xFF
				ih[start + i] = byte_i

		return ih

	def addresses(self):
		return sorted(self._buf.keys())

	def __getitem__(self, x):
		if x in self._buf:
			return self._buf[x]

		return self.fill

	def __setitem__(self, x, val):
		self._buf[x] = val

	@returns("list of defined address ranges", printer=print_ranges, data=True)
	def ranges(self):
		"""
		Get all of the consecutive ranges defined in this file as native
		addresses.
		"""

		valid_addr = self.addresses()
		rstart = valid_addr[0]

		ranges = []

		for i in xrange(1, len(valid_addr)):
			if valid_addr[i] != valid_addr[i-1] + self.skip:
				rend = valid_addr[i-1]
				ranges.append((rstart, rend))
				rstart = valid_addr[i]

		ranges.append((rstart, valid_addr[-1]))

		return ranges

	@param("dest", "path", "writeable", desc='path of output file to create')
	def save(self, dest):
		"""
		Save a copy of the current state of this hexfile to the path specified
		by dest.
		"""

		ih = self._convert_to_hex()
		ih.write_hex_file(dest)

	@param("other", "path", "readable", desc="Other hex file to compare to this one")
	@param("type", "string", ("list", ("oneway, twoway")), desc="compare in both directions")
	@returns("list of mismatches", printer=print_errors, data=True)
	def compare(self, other, type="oneway"):
		"""
		Load another hex file from the given path and compare it to this one.  If type is
		oneway, then only consider the address ranges defined in this file, so the files
		compare as the same if all addresses defined in this HexFile agree with the values
		in the other hex file.  If type is twoway, the comparison is repeated for all address
		ranges defined in the other file as well.
		"""

		if not isinstance(other, HexFile):
			other = HexFile(other, self.wordsize, self.filewords, self.skip, self.cutoff)

		ownranges = self.ranges()

		errors = []

		for r in ownranges:
			for i in xrange(r[0], r[1]+1):
				sval = self[i]
				oval = other[i]

				if sval != oval:
					err = {"range": r, "address": i, "this": sval, "other": oval, "direction": "forward"}
					errors.append(err)
					break

		#if we're asked for a two-way comparison, also compare the other direction
		if type == "twoway":
			rev_errors = other.compare(self)
			for err in rev_errors:
				err['direction'] = 'reverse'

			errors.extend(rev_errors)

		return errors


#price.py
#An object encapsulating the price of something

from decimal import Decimal

two_places = Decimal('0.01')

def convert(arg, **kwargs):
	if isinstance(arg, Decimal):
		return arg

	return Decimal(arg)

def default_formatter(arg, **kwargs):
	return str(arg.quantize(two_places))



#block.py
#An object representing a MIBBlock

from handler import MIBHandler
import sys
import os.path
from pymomo.utilities import build, template
from pymomo.hex8.decode import *
import intelhex

block_size = 16

hw_offset = 0
type_offset = 1
info_offset = 2
name_offset = 3
num_feature_offset = 10
features_offset = 11
cmds_offset = 12
specs_offset = 13
handlers_offset = 14
magic_offset = 15

known_hwtypes = {
	2: "12lf1822",
	3: "16lf1823",
	4: "16lf1847"
}

class MIBBlock:
	"""
	The block in program memory describing a MoMo application module.  The MIB block
	contains information on the application module and a sparse matrix representation
	of a jump table containing all of the feature, command combinations that the 
	module knows how to respond to.
	"""

	TemplateName = 'command_map.asm'

	def __init__(self, ih=None):
		"""
		Given an intelhex object, extract the MIB block information
		from it or raise an exception if a MIBBlock cannot be found
		at the right location.
		"""

		build_settings = build.load_settings()

		self.base_addr = build_settings["mib12"]["mib"]["base_address"]
		self.curr_version = build_settings["mib12"]["mib"]["current_version"]
		self.features = {}
		self.valid = True
		self.error_msg = ""
		self.num_features = 0

		if isinstance(ih, basestring):
			ih = intelhex.IntelHex16bit(ih)

		if ih is not None:
			try:
				self._load_from_hex(ih)
				self._check_consistency()
				self._interpret_handlers()
				self._build_feature_map()
			except ValueError as e:
				self.error_msg = e
				self.valid = False

	def set_variables(self, name, type, flags):
		self.name = name
		self.flags = flags
		self.module_type = type

	def add_command(self, feature, cmd, handler):
		"""
		Add a command to the MIBBlock.  The cmd must be sequential after
		the last command in the feature that it is added to, i.e. you must add
		feature 1, cmd 0 before adding feature 1 cmd 1.
		"""

		if feature not in self.features:
			self.features[feature] = []
			self.num_features = len(self.features.keys())

		if cmd != len(self.features[feature]):
			raise ValueError("Added noncontiguous command %d (%s) to feature %d, last command was %d" % (cmd, handler.symbol, feature, len(self.features[feature])))

		self.features[feature].append(handler)

	def _interpret_handlers(self):
		self.handlers = map(lambda i: MIBHandler(self.spec_list[i], address=self.handler_addrs[i]), xrange(0, len(self.spec_list)))

	def _build_feature_map(self):
		cmd_i  = 0

		self.features = {}
		for i in xrange(0, len(self.feature_list)):
			self.features[self.feature_list[i]] = []
			for j in xrange(self.cmd_list[i], self.cmd_list[i+1]):
				self.features[self.feature_list[i]].append(self.handlers[cmd_i])

				cmd_i += 1

	def _check_magic(self, ih):
		magic_addr = self.base_addr + magic_offset
		instr = ih[magic_addr]

		#Last instruction should be retlw 0xAA for the magic number
		if instr == 0x34AA:
			return True

		return False

	def _check_consistency(self):
		"""
		Check that the features, commands, handlers, and specifications are mutually consistent
		"""

		self.valid = False

		if self.num_features != len(self.feature_list):
			raise ValueError("Mismatch between length of feature list and num_features.")

		if len(self.cmd_list) != len(self.feature_list)+1:
			raise ValueError("Command list has wrong number of entries, was %d, should be %d" % (len(cmds), len(features)+1))

		if len(self.spec_list) != len(self.handler_addrs):
			raise ValueError("There should be a 1-1 mapping between command specs and handlers. Their lengths differed (specs: %d, handlers: %d)." % (len(self.specs), len(self.handler_addrs)))

		if len(self.handler_addrs) != self.cmd_list[-1]:
			raise ValueError("Command array has invalid format.  Last entry should be the number of total handlers (total handlers: %d, cmd entry: %d)" % (len(self.handler_addrs), self.cmds[-1]))

		self.valid = True

	def _load_from_hex(self, ih):
		if not self._check_magic(ih):
			raise ValueError("Invalid magic number.")

		self.num_features = decode_retlw(ih, self.base_addr + num_feature_offset)
		self.features_addr = decode_goto(ih, self.base_addr + features_offset)
		self.cmds_addr = decode_goto(ih, self.base_addr + cmds_offset)
		self.specs_addr = decode_goto(ih, self.base_addr + specs_offset)
		self.handlers_addr = decode_goto(ih, self.base_addr + handlers_offset)

		self.feature_list = decode_table(ih, self.features_addr, decode_retlw)
		self.cmd_list = decode_table(ih, self.cmds_addr, decode_retlw)
		self.spec_list = decode_table(ih, self.specs_addr, decode_retlw)
		self.handler_addrs = decode_table(ih, self.handlers_addr, decode_goto)

		self.name = decode_string(ih, self.base_addr + name_offset, 7)
		self.hw_type = decode_retlw(ih, self.base_addr + hw_offset)
		self.info = decode_retlw(ih, self.base_addr + info_offset)
		self.module_type = decode_retlw(ih, self.base_addr + type_offset)

		self.revision = self.info >> 4
		self.flags = self.info & 0x0F

		self._parse_hwtype()

	def _parse_hwtype(self):
		"""
		Convert the numerical hw id to a chip name using the well-known
		conversion table
		"""

		if self.hw_type not in known_hwtypes:
			self.chip_name = "Unknown Chip (type=%d)" % self.hw_type

		self.chip_name = known_hwtypes[self.hw_type]


	def create_asm(self, folder):
		temp = template.RecursiveTemplate(MIBBlock.TemplateName)
		temp.add(self)
		temp.render(folder)

	def __str__(self):
		rep = "\n"

		rep += "MIB Block\n"
		rep += "---------\n"

		if not self.valid:
			rep += "Block invalid: %s\n" % self.error_msg
			return rep

		rep += "Block Valid\n"
		rep += "Name: '%s'\n" % self.name
		rep += "Type: %d\n" % self.module_type
		rep += "Hardware: %s\n" % self.chip_name
		rep += "MIB Revision: %d\n" % self.revision
		rep += "Flags: %s\n" % bin(self.flags)
		rep += "Number of Features: %d\n" % self.num_features

		rep += "\nFeature List\n"



		for f in self.features.keys():
			rep += "%d:\n" % f
			for i,h in enumerate(self.features[f]):
				rep += "    %d: %s\n" % (i, str(h))

		return rep


#integer type

def convert(arg):
	if arg is None:
		return None

	if isinstance(arg, basestring):
		return int(arg, 0)
	elif isinstance(arg, int) or isinstance(arg, long):
		return arg
	
	raise TypeError("Unknown argument type")

#Validation Functions
def validate_positive(arg):
	if arg is None:
		return

	if arg <=0:
		raise ValueError("value is not positive")

def validate_nonnegative(arg):
	if arg is None:
		return

	if arg < 0:
		raise ValueError("value is negative")

def validate_range(arg, lower, upper):
	if arg is None:
		return

	if arg < lower or arg > upper:
		raise ValueError("not in required range [%d, %d]" %(int(lower), int(upper)))

#Formatting functions
def default_formatter(arg, **kwarg):
	return str(arg)

def format_unsigned(arg, **kwarg):
	return format(arg, 'd')

def format_hex(arg, **kwarg):
	return "0x%X" % arg


#simworker.py
#Base class for worker objects that implement PIC simulators
from pymomo.exceptions import *
import time
import select

class SimWorker:
	def __init__(self, pipe):
		self.pipe = pipe
		self._build_sim()

	def _build_sim(self):
		raise APIError("SimWorker subclasses must override _build_sim()")

	def _build_message(self, error=False, message=None, **kwargs):
		msg = {
			'error': error,
			'message': message,
		}

		msg.update(kwargs)
		return msg

	def _format_error(self, message, **kwargs):
		return self._build_message(error=True, message=message, **kwargs)


	def _check_cmd(self, cmd):
		if not isinstance(cmd, dict):
			raise APIError("Command packet is not a dictionary")

		if "method" not in cmd:
			raise APIError("No method specified in command packet")

	def send_error(self, message, **kwargs):
		msg = self._format_error(message, **kwargs)
		self.pipe.send(msg)

	def send_message(self, **kwargs):
		msg = self._build_message(**kwargs)
		self.pipe.send(msg)

	def _dispatch_cmd(self, cmd):
		"""
		Map the command to one of our methods.  Return False if quit
		is invoked
		"""
		method = cmd['method']

		if method == 'quit':
			self.pipe.send(self._build_message(message="Quit command successful"))
			return False

		#If the method exists, call it and return the results
		if hasattr(self, method):
			params = cmd.copy()
			del params['method']

			result = getattr(self, method)(**params)
			if result is not None:
				self.send_message(**result)
			else:
				self.send_message()
		else:
			raise APIError('Method does not exist: %s' % method)

		return True

	def run(self):
		"""
		Run forever in a loop until we receive a quit command.
		"""

		while(True):
			try:
				cmd = self.pipe.recv()
				self._check_cmd(cmd)

				if self._dispatch_cmd(cmd) is False:
					self.finish()
					return
			except MoMoException as e:
				#All errors are propogated as exceptions, catch them and 
				#send an error report.

				msg = self._format_error(e.msg, **e.params)
				self.pipe.send(msg)
			except Exception as ex:
				args = ex.args
				if len(args) == 1:
					args = args[0]

				msg = self._format_error(args)
				self.pipe.send(msg)
			except KeyboardInterrupt as ex:
				self.finish()

	def finish(self):
		"""
		Cleanup all the resources associated with this simulator instance.
		Subclasses should override this behavior
		"""

		pass



#config12.py

from pymomo.utilities import build

class MIB12Processor:
	"""
	Collection of settings pertaining to the location of various important
	ROM and RAM regions in the MIB12 framework.
	"""

	def __init__(self, name, settings):
		self.settings = settings
		self.name = name

		self._calculate()

	def _calculate(self):
		self.exec_rom = self.settings['executive_rom']
		self.app_rom = [self.exec_rom[1] +1, self.settings['total_rom']-1]

		self.api_range = [self.app_rom[0] - 16, self.app_rom[0] - 1]
		self.mib_range = [2048 - 16, 2047] #mib map is 16 bytes long

		self.row_size = self.settings['flash_row_words']
		self.total_prog_mem = self.settings['total_rom']
		self.first_app_page = self.app_rom[0] / self.row_size

	@classmethod
	def FromChip(cls, chip):
		"""
		Create a processor instance from a valid chip name defined in build_settings.json
		"""

		fam = build.ChipFamily('mib12')
		info = fam.find(chip)

		return MIB12Processor(chip, info.settings)


#descriptor.py
#A DSL for specifying log entries that can be used with the MoMo syslog module
#to log and debug system events in hardware.  Each log entry has a short message
#an optional description and an optional list of typed data items that follow it.

import pyparsing
from pyparsing import Word, Regex, nums, Literal, Optional, Group, QuotedString
from pymomo.exceptions import *
from pymomo.utilities.typedargs.annotate import context, param
from logdefinition import LogDefinition
from pymomo.utilities.template import RecursiveTemplate
import shutil
import os.path

symbol = Regex('[_a-zA-Z][_a-zA-Z0-9]*')

log_def = symbol("name")
log_text = QuotedString('"')
attribute_def = Literal('-').suppress() + QuotedString('"') + Literal('as').suppress() + Optional(Literal('list of')) + symbol('type') + Optional(Literal(',').suppress() + Literal("format").suppress() + symbol('format'))

statement = log_def | log_text | attribute_def

@context()
class LogDefinitionMap:
	"""
	A map defining known logging statement definitions.

	LogDefinitionMap contains a list of defined MoMo system_log logging statement types,
	including how to interpret the arguments that accompany the statement. This object
	is built by interpreting one or more Log Definition Files (.ldf).
	"""

	def __init__(self):
		self.entries = {}
		self.sources = []

	@param("file", "path", "readable", desc='Path of an ldf file to parse and add to this map')
	def add_ldf(self, file):
		"""
		Parse and add the specified LDF file.

		Adds the file to this LogDefinitionMap, throwing an exception if
		there is a collision with existing definitions.
		"""

		with open(file, "r") as f:
			lines = f.readlines()

		current_definition = None
		for line in lines:
			line = line.strip()
			if line == '':
				continue

			try:
				match = statement.parseString(line)

				#If we are starting a new statement and we have an existing one, try 
				#to add it to the map
				if 'name' in match:
					if current_definition is not None:
						self._add_definition(current_definition)

					current_definition = LogDefinition()
					current_definition.name = match['name']
				elif 'type' in match:
					offset = 0
					is_list = False
					if match[1] == 'list of':
						offset = 1
						is_list = True

					name = match[0]
					type = match[1+offset]
					format = None
					if "format" in match:
						format = match['format']

					if current_definition is None:
						raise InternalError("Invalid data definition outside of LogDefinition", line=line)

					if current_definition.contains_list():
						raise InternalError("You cannot specify a data definition after a list since lists have arbitrary length", line=line)
						
					current_definition.add_data(name, type, format, is_list=is_list)
				else:
					if current_definition is None:
						raise InternalError("Invalid log message outside of LogDefinition", line=line)

					current_definition.message = match[0]
			except pyparsing.ParseException as e:
				raise InternalError("Error parsing ldf file", file=file, line=e.lineno, column=e.col, reason=str(e))

		if current_definition is not None:
			self._add_definition(current_definition)

		self.sources.append(file)

	def _add_definition(self, statement):
		"""
		Verify the LogDefinition and add it to the map
		"""

		valid, reason = statement.validate()
		if not valid:
			raise InternalError("Invalid or incomplete LogDefinition", reason=reason, definition=statement)

		h = statement.hash()
		if h in self.entries:
			raise InternalError("Two log entry definitions hash to the same value, they must be distinct", 	first=self.entries[h].name, 
																											second=statement.name)
		self.entries[h] = statement

	@param("file", "path", "writeable", desc='output location for the header file')
	def generate_header(self, file):
		"""
		Generate a header file with these log statements
		"""

		name = os.path.basename(file)
		name = name.replace(' ', '_').replace('.', '_')

		vals = {"k"+v.name: hex(h)+"UL"  for h,v in self.entries.iteritems()}

		templ = RecursiveTemplate('logdefinitions.h')
		templ.add({'messages': vals, 'sources': self.sources, 'name': name})
		out = templ.format_temp()

		shutil.move(out, file)

	@param("id", "integer", desc='the hashed message id to map')
	def map(self, id):
		"""
		Map a message id to its descriptive string and parameter list
		"""

		if id in self.entries:
			return self.entries[id]

		unknown = LogDefinition()
		unknown.name = "0x" + format(id, "08X")
		return unknown


#basic tests to ensure that everything is installed and working properly

def test_required_packages():
	"""
	Make sure that the required dependencies of pymomo are installed
	"""

	import decorator
	#TODO add the rest of the packages

def test_import_pymomo():
	"""
	Make sure pymomo is accessible for import
	"""

	import pymomo


import proxy12
from pymomo.commander.exceptions import *
from pymomo.commander.types import *
from pymomo.commander.cmdstream import *
from pymomo.utilities.console import ProgressBar
import struct
from intelhex import IntelHex
from time import sleep

class GSMModule (proxy12.MIB12ProxyObject):
	def __init__(self, stream, addr):
		super(GSMModule, self).__init__(stream, addr)
		self.name = 'GSM Module'

	def power_module(self, on=True):
		"""
		Turn on power to the GSM module
		"""

		if on:
			self.rpc(10, 1)
		else:
			self.rpc(10, 6)

	def send_message(self, destination, text):
		"""
		Send a  message to the given destination which must have the form:
		+NUMBER with no dashes or spaces, for example: +16506695211, or a URL
		"""
		apn = "JTM2M"

		print "Sending message '%s' to %s" % ( text, destination )
		print "> apn %s" % apn
		self.rpc(10, 9, apn)
		for i in xrange(0, len(destination), 18):
			buf = destination[i:i+18]
			print "> comm_destination %d %s" % ( i, buf )
			self.rpc(11, 4, i, buf)
		print "> start %d" % len(text)
		self.rpc(11, 0, len(text))
		for i in xrange(0, len(text), 20):
			buf = text[i:i+20]
			print "> stream (%s)" % buf
			self.rpc(11, 1, buf)
		print "> end"
		self.rpc(11, 2)

	def module_on(self):
		res = self.rpc(10, 0, result_type=(1, False))

		if res['ints'][0] == 1:
			return True

		return False

	def module_off(self):
		res = self.rpc(10,5)
		return True

	def at_cmd(self, cmd, wait=0.5):
		"""
		Send an AT command to the GSM module and wait for its
		response.
		"""

		res = self.rpc(10, 2, cmd, result_type=(0, True))

		return res['buffer']

	def dump_buffer(self):
		output = ""
		while True:
			res = self.rpc(10,4, result_type=(0, True))
			if len(res['buffer']) == 0:
				break;
			output += res['buffer']
		return output

	def debug(self):
		res = self.rpc(10,7, result_type=(0, True))

		return res['buffer']

	def set_apn(self,apn):
		res = self.rpc(10,9,apn)

	def test_gprs(self):
		apn = "wap.cingular"

		print "> apn %s" % apn
		self.rpc(10, 9, apn)
		print "> testgprs"
		res = self.rpc(10,8, result_type=(0, True))

		return res['buffer']



import proxy
import proxy12
from pymomo.commander.exceptions import *
from pymomo.commander.types import *
from pymomo.commander.cmdstream import *
from pymomo.utilities.console import ProgressBar
import struct
from intelhex import IntelHex
from time import sleep
from datetime import datetime
from pymomo.syslog import RawLogEntry, SystemLog
from pymomo.utilities.typedargs.annotate import annotated,param,returns, context
from pymomo.utilities import typedargs
from pymomo.exceptions import *
import itertools
import base64
import uuid

#Formatters for the return types used in this class
def print_module_list(mods):
	print "Listing attached modules"
	for i, mod in enumerate(mods):
		print "%d: %s at address %d" % (i, mod.name, mod.address)

def _print(x):
	print x

@context("MIBController")
class MIBController (proxy.MIBProxyObject):
	"""
	A python proxy object for interacting with the MoMo controller board.
	"""

	MaxModuleFirmwares = 4

	@annotated
	def __init__(self, stream):
		super(MIBController, self).__init__(stream, 8)
		self.name = 'Controller'

	@returns(desc='number of attached module', data=True)
	def count_modules(self):
		"""
		Count the number of attached devices to this controller
		"""

		res = self.rpc(42, 1, result_type=(1, False))
		return res['ints'][0]

	def reflash(self):
		try:
			self.rpc(42, 0xA)
		except RPCException as e:
			if e.type != 7:
				raise e

	def describe_module(self, index):
		"""
		Describe the module given its module index
		"""
		res = self.rpc(42, 2, index, result_type=(0,True))

		return ModuleDescriptor(res['buffer'], 11+index)

	@annotated
	def clear_log(self):
		"""
		Clear the MoMo system log.
		"""

		self.rpc(42,0x23)

	@annotated
	def read_log(self):
		"""
		Read the MoMo system log
		"""
		count = self.rpc(42, 0x21, result_type=(1,False))['ints'][0]

		log = []

		for i in xrange(0, count):
			res = self.rpc(42, 0x22, i, 0, result_type=(0, True))
			try:
				log.append(RawLogEntry(res['buffer']))
			except ValidationError as e:
				print "FAILED TO PARSE A LOG ENTRY, %d entries discarded" % (count - i)
				break

		return SystemLog(log)

	@param("name", "string", desc="module name")
	@param("address", "integer", ("range", 11, 127), desc="modules address")
	def get_module(self, name=None, address=None, force=False):
		"""
		Given a module name or a fixed address, return a proxy object
		for that module if it is connected to the bus.  If force is True
		this call will construct an proxy object for the given address even
		if the module is not found in the controller's database of connected
		modules.
		"""

		if address is not None and force:
			obj = proxy12.MIB12ProxyObject(self.stream, address)
			obj.name = 'Unknown'
			return obj

		mods = self.enumerate_modules()
		if name is not None and len(name) < 7:
			name += (' '*(7 - len(name)))

		for mod in mods:
			if name == mod.name or address == mod.address:
				obj = proxy12.MIB12ProxyObject(self.stream, mod.address)
				obj.name = mod.name
				return obj

		raise ValueError("Could not find module by name or address (name=%s, address=%s)" % (str(name), str(address)))

	@returns(desc='list of attached modules', printer=print_module_list, data=True)
	def enumerate_modules(self):
		"""
		Get list of all attached modules and describe them all
		"""
		
		num = self.count_modules()

		mods = []

		for i in xrange(0, num):
			mods.append(self.describe_module(i))

		return mods

	def get_debug_value(self):
		"""
		Return the current value of the debug flag in the controller.
		This is an unsigned integer that can be used to signal that 
		processes are ocurring or not occuring as they should.
		"""

		res = self.rpc(42, 0x0D, result_type=(1, False))
		flag = res['ints'][0]
		return flag

	def set_sleep(self, val):
		"""
		Enable or disable sleeping on the controller when there are no tasks 
		to perform.  val should be True to enable sleeping, False to disable 
		it. 
		"""
		res = self.rpc(42, 0x0E, int(bool(val)))

	def battery_status(self):
		res = self.rpc(42, 0x0B, result_type=(1, False))
		volt_raw = res['ints'][0]

		volt = volt_raw/1024. * 2.78 * 2.0 	#ADC is on a divide by 2 resistor network with a VCC reference
		return volt

	def _encode_firmware_line(self, line):
		if line[0] != ':':
			raise ValueError("Invalid hex line, did not start with a ':': %s" % line)

		line = line[1:]

		cnt = int(line[0:2], 16)
		addr = int(line[2:6], 16)
		rec = int(line[6:8], 16)

		if cnt > 16:
			raise ValueError("Cannot use hex file with more than 16 bytes per line. Line size was %d." % cnt)

		datahex = line[8:8+2*cnt]
		chk = int(line[8+2*cnt:8+2*cnt+2], 16)

		data = bytearray(cnt)
		for i in xrange(0, cnt):
			data[i] = int(datahex[2*i:2*i+2], 16)

		packed = struct.pack('HB%dB' % cnt, addr, rec, *data)
		return bytearray(packed)

	def push_firmware(self, firmware, module_type, verbose=True):
		"""
		Given either a path to a hex file or an open file-like object,
		push the firmware described by the object to the controller.

		returns an integer, which is the firmware bucket that this
		firmware was pushed to.
		"""

		lines = []

		if hasattr(firmware, "readlines"):
			lines = firmware.readlines()
		elif isinstance(firmware, basestring):
			with open(firmware, "r") as f:
				lines = f.readlines()

		if len(lines) == 0:
			raise ValueError("Could not understand firmware object type.  Should be a path or a file object")


		if verbose:
			print "Sending Firmware"
			prog = ProgressBar("Transmission Progress", len(lines))
			prog.start()

		res = self.rpc(7, 0, module_type, result_type=(1, False))
		bucket = res['ints'][0]

		for i in xrange(0, len(lines)):
			buf = self._encode_firmware_line(lines[i])
			res = self.rpc(7, 1, buf)

			if verbose:
				prog.progress(i+1)

		if verbose:
			prog.end()
			print "Firmware stored in bucket %d" % bucket

		return bucket

	def pull_firmware(self, bucket, verbose=True, pic12=True):
		res, reason = self._firmware_bucket_loaded(bucket)

		if res == False:
			raise ValueError(reason)

		info = self.get_firmware_info(bucket)
		length = info['length']

		out = IntelHex()

		if verbose:
			print "Getting Firmware, size=0x%X" % length
			prog = ProgressBar("Transmission Progress", length // 20)
			prog.start()

		for i in xrange(0, length, 20):
			res = self.rpc(7, 4, bucket, i, result_type=(0, True))

			for j in range(0, len(res['buffer'])):
				out[i+j] = ord(res['buffer'][j])
				if pic12 and (j%2 != 0):
					out[i+j] &= 0x3F

			if verbose:
				prog.progress(i//20)

		if verbose:
			prog.end()

		return out

	def _firmware_bucket_loaded(self, index):
		res = self.get_firmware_count()
		mods = res['module_buckets']

		if index < 4 and index >= mods:
			return False, "Invalid firmware bucket specified, only %d buckets are filled" % mods

		if index == 4 and not res['controller_firmware']:
			return False, "Controller firmware requested and none is loaded"

		if index == 5 and not res['backup_firmware']:
			return False, "Backup firmware requested and none is loaded"

		if index > 5:
			return False, "Invalid bucket index, there are only 6 firmware buckets (0-5)."

		return True, "" 

	def get_firmware_count(self):
		"""
		Return a dictionary containing:
		- the number of module firmwares loaded
		- if there is a controller firmware loaded
		- if there is a backup controller firmware loaded
		"""

		res = self.rpc(7, 5, result_type=(2, False))
		con = res['ints'][1]

		dic = {}
		dic['module_buckets'] = res['ints'][0]
		dic['controller_firmware'] = (con&0b1 == 1)
		dic['backup_firmware'] = ((con >> 1) == 1)

		return dic

	def clear_firmware_cache(self):
		self.rpc(7, 0x0A)

	def get_firmware_info(self, bucket):
		"""
		Get the firmware size and module type stored in the indicated bucket
		"""

		res = self.rpc(7, 3, bucket, result_type=(7, False))

		length = res['ints'][1] << 16 | res['ints'][2]
		base = res['ints'][3] << 16 | res['ints'][4]
		sub_start = res['ints'][5]
		subs = res['ints'][6]

		return {'module_type': res['ints'][0], 'length': length, 'base_address': base, 'bucket_start': sub_start, 'bucket_size': subs}

	def read_flash(self, baseaddr, count, verbose=True):
		"""
		Read count bytes of the controller's external flash starting at addr.
		The flash is read 20 bytes at a time since this is the maximum mib 
		frame size.
		"""

		if baseaddr >= 1024*1024:
			raise ValueError("Address too large, flash is only 1MB in size")

		buffer = bytearray()

		if verbose:
			prog = ProgressBar("Reading Flash", count)
			prog.start()

		for i in xrange(0, count, 20):
			if verbose:
				prog.progress(i)

			addr = baseaddr + i
			addr_low = addr & 0xFFFF
			addr_high = addr >> 16

			length = min(20, count-i)

			res = self.rpc(42, 3, addr_low, addr_high, result_type=(0, True))
			buffer.extend(res['buffer'][0:length])

		if len(buffer) != count:
			raise RuntimeError("Unknown error in read_flash, tried to read %d bytes, only read %d" % (count, len(buffer)))

		if verbose:
			prog.end()

		return buffer

	def write_flash(self, baseaddr, buffer, verbose=True):
		"""
		Write all of the bytes in buffer to the controller's external flash starting at baseaddr
		"""

		count = len(buffer)

		if baseaddr >= 1024*1024:
			raise ValueError("Address too large, flash is only 1MB in size")

		if verbose:
			prog = ProgressBar("Writing Flash", count)
			prog.start()

		i = 0
		while i < count:
			if verbose:
				prog.progress(i)

			addr = baseaddr + i
			addr_low = addr & 0xFFFF
			addr_high = addr >> 16

			length = min(14, count-i)		
			res = self.rpc(42, 4, addr_low, addr_high, buffer[i:i+length], result_type=(2, False))

			written_addr = res['ints'][0] | (res['ints'][1] << 16)

			if written_addr != addr:
				raise RuntimeError("Tried to write to address 0x%X but wrote to address 0x%X instead" % (addr, written_addr))

			i += length

		if verbose:
			prog.end()

	def erase_flash(self, addr):
		if addr >= 1024*1024:
			raise ValueError("Address too large, flash is only 1MB in size")

		addr_low = addr & 0xFFFF
		addr_high = addr >> 16

		res = self.rpc(42, 6, addr_low, addr_high)

	def bootloader_status(self):
		"""
		Check to see if the alarm pin is blinking at a fixed rate.  This is the
		bootloader's way of communicating with us, so figure out the pulse freq
		and match it to the bootloader's error code.
		"""

		alarm_codes = {
			100: "Invalid firmware magic number",
			150: "Invalid firmware checksum",
			200: "Invalid metadata checksum",
			250: "Hardware compatibility version does not match",
			300: "Address error (internal bootloader error)",
			350: "No firmware image loaded",
			400: "Wrong firmware length"
		}

		probe = []

		val = False
		for i in xrange(0, 400):
			down = self.alarm_asserted()
			if down != val:
				t = datetime.now()
				val = down
				probe.append(t)

			sleep(0.01)

		trans_pairs = itertools.izip(probe[2:], probe[1:-1])
		widths = map(lambda x: (x[0] - x[1]).total_seconds()*1000.0, trans_pairs)
		
		if len(widths) == 0:
			return "No Bootloader Error Detected"

		avg_width = sum(widths)/len(widths)

		code = min(alarm_codes.keys(), key=lambda x:abs(x-avg_width))

		return "Bootloader Error Detected: %s" % alarm_codes[code]


	def alarm_asserted(self):
		"""
		Query the field service unit if the alarm pin on the bus is asserted.  Returns True if 
		the alarm is asserted (low value since it's active low).  Returns False otherwise
		"""

		resp, result = self.stream.send_cmd("alarm status")
		if result != CMDStream.OkayResult:
			raise RuntimeError("Alarm status command failed")

		resp = resp.lstrip().rstrip()
		val = int(resp)

		if val == 0:
			return True
		elif val == 1:
			return False
		else:
			raise RuntimeError("Invalid result returned from 'alarm status' command: %s" % resp)


	def set_alarm(self, asserted):
		"""
		Instruct the field service unit to assert or deassert the alarm line on the MoMo bus.
		"""

		if asserted:
			arg = "yes"
		else:
			arg = "no"

		cmd = "alarm %s" % arg

		resp, result = self.stream.send_cmd(cmd)
		if result != CMDStream.OkayResult:
			raise RuntimeError("Set alarm command failed")

	@returns(desc='bluetooth receive buffer', data=True)
	def bt_debug_log(self):
		"""
		Return the first 20 bytes of BTLE communication receive buffer.
		This contains the response to the last command sent to the unit.
		"""


		res = self.rpc(42, 0x27, result_type=(0,True))
		return str(res['buffer'])

	def momo_attached(self):
		resp, result = self.stream.send_cmd("attached")
		if result != CMDStream.OkayResult:
			raise RuntimeError("Attached command failed")

		resp = resp.lstrip().rstrip()
		val = int(resp)

		if val == 1:
			return True
		elif val == 0:
			return False
		else:
			raise RuntimeError("Invalid result returned from 'attached' command: %s" % resp)

	def sensor_log( self, stream, meta, value):
		res = self.rpc( 70, 0, stream, meta, struct.pack( 'Q', value ) );

	def sensor_log_read( self ):
		res = self.rpc( 70, 0x1, result_type=(0, True) )
		return SensorEvent( res['buffer'] )

	def sensor_log_count( self ):
		res = self.rpc( 70, 0x2, result_type=(0, True) )
		(count, ) = struct.unpack('I', res['buffer'])
		return count

	def sensor_log_clear( self ):
		res = self.rpc( 70, 0x3 )

	def sensor_log_debug( self ):
		res = self.rpc( 70, 0x4, result_type=(0, True) )
		(min, max, start, end) = struct.unpack('IIII', res['buffer'])
		return (min,max,start,end)

	def start_reporting(self):
		"""
		Start regular reporting
		"""
		self.rpc(60, 1)

	def stop_reporting(self):
		"""
		Stop regular reporting
		"""
		self.rpc(60, 2)

	def get_reporting(self):
		"""
		Get regular reporting state
		"""
		res = self.rpc(60,14, result_type=(1, True))
		enabled = bool(res['ints'][0])
		return enabled

	def reset_reporting_config(self):
		"""
		Reset the reporting configuration.
		"""
		res = self.rpc(60,0x12)

	def send_report(self):
		"""
		Send a single report
		"""

		self.rpc(60, 0)

	def set_report_route(self, index, route):
		if len(route) == 0:
			arg = struct.unpack('H', struct.pack('BB', 0, int(index)))
			self.rpc(60, 5, arg[0], route)
		else:
			for i in xrange(0, len(route), 18):
				buf = route[i:i+18]
				arg = struct.unpack('H', struct.pack('BB', int(i), int(index)))
				self.rpc(60, 5, arg[0], buf)

	def get_report_route(self, index):
		route = ""
		i = 0
		while True:
			arg = struct.unpack('H', struct.pack('BB', int(i),int(index)))
			res = self.rpc(60, 6, arg[0], result_type=(0, True))
			if len(res['buffer']) == 0:
				break
			i += len(res['buffer']);
			route += res['buffer']
		return route

	def get_gprs_apn(self):
		res = self.rpc(60, 0x14, result_type=(0,True))
		return res['buffer']

	def set_gprs_apn(self, apn):
		res = self.rpc(60, 0x13, apn)

	@annotated
	@returns(desc="Time", data=True)
	def current_time(self):
		"""
		Get the current time according to the controller's RTCC
		"""

		res = self.rpc(42, 0x0C, result_type=(0, True));
		year, month, day, hour, minute, second = struct.unpack( "HHHHHH", res['buffer'] )

		return datetime( year+2000, month, day, hour, minute, second )

	@annotated
	def set_time(self, year, month, day, hours, minutes, seconds, weekday=0 ):
		"""
		Set the current time of the controller's RTCC
		"""

		packed = struct.pack('BBBBBBBB', int(year), int(month), int(day), int(hours), int(minutes), int(seconds), int(weekday), 0);
		self.rpc(42, 0x12, packed)

	@annotated
	def sync_time(self):
		"""
		Set the time on the momo controller to the current system time.
		"""

		now = datetime.now()
		self.set_time( now.date().year - 2000, now.date().month, now.date().day, now.time().hour, now.time().minute, now.time().second, now.weekday() )
		print now

	@annotated
	def build_report(self):
		res = self.rpc(60, 0x0C)

	@annotated 
	def get_report(self):
		report = ""
		for i in xrange(0, 160, 20):
			res = self.rpc(60, 0x0D, i, result_type=(0, True))
			report += res['buffer']

		print report

	@param('counter', 'integer', 'nonnegative')
	@returns(desc="Performance Counter", printer=lambda x: x.display(), data=True)
	def perf_counter(self, counter):
		res = self.rpc(42, 0x26, counter, result_type=(0, True))
		return PerformanceCounter(res['buffer'], name=counter)

	@param('address', 'integer', 'nonnegative', desc='address to start reading')
	@param('type', 'string', desc='format read data as this type')
	@param('size', 'integer', 'nonnegative', desc='optional size of data to read')
	@returns(desc='read object')
	def read_ram(self, address, type=None, size=0):
		"""
		Read bytes from the controller ram starting at the specified address.  If a 
		type is provided, use the type information to determine how many bytes to read
		and how to format the results.  If type is supplied, an object of that type is
		returned. Otherwise read size bytes are read and returned as a binary string
		object.
		"""

		if type is not None:
			if not typedargs.is_known_type(type):
				raise ArgumentError("unknown type specified", type=type)

			if size == 0:
				size = typedargs.get_type_size(type)

		if size == 0:
			raise ArgumentError("size not specified and could not be determined from supplied type info", supplied_size=size, supplied_type=type)

		buf = ""

		for i in xrange(0, size, 20):
			res = self.rpc(42, 0x11, address+i, result_type=(0, True))
			valid = min(20, size - i)
			buf += res['buffer'][:valid]
		
		if size != len(buf):
			raise InternalError("Read size does not match specified size, this should not happen", read_size=len(buf), specified_size=size)
		
		if type is not None:
			return typedargs.convert_to_type(buf, type)

		return buf

	def scheduler_map(self):
		"""
		Get the map of used scheduler buckets
		"""

		res = self.rpc(43, 2, result_type=(1, False));

		return res['ints'][0];

	def scheduler_new(self, address, feature, command, frequency):
		"""
		Schedule a new task
		"""

		res = self.rpc(43, 0, int(address), ( (int(feature)<<8) | (int(command)&0xFF) ), int(frequency) );

	def scheduler_remove(self, address, feature, command, frequency):
		"""
		Remove a scheduled task
		"""

		res = self.rpc(43, 1, int(address), ( (int(feature)<<8) | (int(command)&0xFF) ), int(frequency) );

	def scheduler_describe(self, index):
		"""
		Describe a scheduled callback
		"""

		try:
			res = self.rpc(43, 3, int(index), result_type=(0,True) );
			return struct.unpack('BBBxB', res['buffer'][:5])
		except RPCException as e:
			if e.type == 7:
				return None

	@annotated
	def reset(self, sync=True):
		"""
		Instruct the controller to reset itself.
		"""

		try:
			self.rpc(42, 0xF)
		except RPCException as e:
			if e.type != 7:
				raise e

		if sync:
			sleep(1.5)

	@annotated
	@returns(desc="Module UUID", printer=_print, data=True)
	def uuid(self):
		"""
		Get the current UUID of the controller.
		"""

		res = self.rpc(42, 0x13, result_type=(2,False))
		guid = struct.pack('HH', res['ints'][0], res['ints'][1])
		print struct.unpack('<L', guid)[0];
		return base64.b64encode(bytes(guid)).rstrip('=')

	@annotated 
	def register(self):
		#TODO: register with the server
		guid = uuid.uuid4()
		guid = struct.unpack('<L', guid.bytes[-4:])[0]
		print guid
		print base64.b64encode(struct.pack('<L', guid)).rstrip('=')
		self.rpc(42, 0x14, guid & 0xFFFF, guid >> 16)

	@annotated
	def factory_reset(self):
		"""
		Instruct the controller to reset all internal state to "factory defaults."  Use with caution.
		"""
		self.rpc(42, 0x10)


#reflash.py
#methods to build a mib12_executive reflashing hex file

from pymomo.hex8.ranges import *
from pymomo.hex8.merge import *
from pymomo.hex8.decode import *
from pymomo.hex8.instruction import Instruction
from config12 import MIB12Processor
import math

def build_reflasher(stub, payload, chip):
	"""
	Given two intelhex16 objects specifying the stub and payload files,
	combine them into one object making intelligent choices about offsets
	and sizes based on the chip information.

	Be very verbose so that errors don't get hidden from the user.
	"""

	proc = MIB12Processor.FromChip(chip)
	stub_info = analyze_stub(stub, proc)
	load_info = analyze_payload(payload, proc)

	if load_info['load_size'] > stub_info['max_size']:
		raise ValueError("Payload is too big.  Would exceed chip memory")

	insert_hex(stub, payload, start_addr=stub_info['start'], skip_ranges=stub_info['skip'], payload_size=load_info['load_size'])

	#patch the stub payload size and start
	start_row = stub_info['start'] / proc.row_size
	size_row = load_info['load_size'] / proc.row_size

	start_addr = proc.app_rom[0] + 1
	size_addr = proc.app_rom[0] + 2

	oldret = Instruction('retlw 0')

	if stub[start_addr] != Instruction('retlw 0xAB').encode():
		raise ValueError("Invalid reflashing stub, wrong instruction at start_addr 0x%X" % start_addr)

	if stub[size_addr] != Instruction('retlw 0xCD').encode():
		raise ValueError("Invalid reflashing stub, wrong instruction at size_addr 0x%X" % size_addr)

	start_instr = 'retlw 0x%X' % start_row
	size_instr = 'retlw 0x%X' % size_row

	print "Patching start instruction: %s" % start_instr
	stub[start_addr] = Instruction(start_instr).encode()

	print "Patching size instruction: %s" % size_instr
	stub[size_addr] = Instruction(size_instr).encode()

def extract_reflasher(stub, chip):
	proc = MIB12Processor.FromChip(chip)
	start_instr = proc.app_rom[0] + 1
	size_instr = proc.app_rom[0] + 2

	start_row = decode_retlw(stub, start_instr)
	size_row = decode_retlw(stub, size_instr)

	start_addr = start_row*proc.row_size
	load_size = size_row*proc.row_size

	print "Extracted starting address: 0x%X" % start_addr
	print "Extracted payload size: %d" % load_size

	mib_start = 2048 - proc.row_size
	mib_end = proc.mib_range[1]
	skip = build_range(mib_start, mib_end)

	print "Skipping over: (0x%X, 0x%X)" % (skip[0][0],skip[0][1])

	return extract_hex(stub, start_addr=start_addr, skip_ranges=[skip], payload_size=load_size+32)

def analyze_stub(stub, proc):
	print "\nAnalyzing Stub"
	print "Chip type: ", proc.name

	ranges = extract_ranges(stub)
	mibblock = proc.mib_range

	found_mib = False 
	highest_addr = 0

	print "Defined ranges:"
	for r in ranges:
		if r[0] == mibblock[0] and r[1] == mibblock[1]:
			print "[0x%X-0x%X] <--- MIB Block (skipping)" % (r[0], r[1])
			found_mib = True
		else:
			highest_addr = r[1]
			print "[0x%X-0x%X]" % (r[0], r[1])

	if not found_mib:
		raise ValueError("Could not extract mib_block from stub file. Bailing")

	mib_start = int(math.floor(mibblock[0]/proc.row_size))*proc.row_size
	mib_end = mibblock[1]

	if (highest_addr + 1) % proc.row_size != 0:
		raise ValueError("Stub does not end on a row boundary, cannot insert payload")

	print "Skipping mib row: (0x%X, 0x%X)" % (mib_start, mib_end)
	print "Starting Address for payload: 0x%X" % (highest_addr+1)
	
	skip = build_range(mib_start, mib_end)
	payload_start = highest_addr + 1

	max_size = proc.total_prog_mem - payload_start - proc.row_size #we can fill the entire memory - the mib block row
	print "Maximum payload size: %d words" % max_size

	return {'skip':[skip], 'start':payload_start, 'max_size': max_size} 

def analyze_payload(load, proc):
	print "\nAnalyzing Payload"

	ranges = extract_ranges(load)

	highest_addr = 0

	print "Defined ranges:"
	for r in ranges:
		if r[0] >= (1<<15):
			print "[0x%X-0x%X] <--- Config Words (ignored)" % (r[0], r[1])
		else:
			highest_addr = r[1]
			print "[0x%X-0x%X]" % (r[0], r[1])

	#The actual bootloader is 2 words smaller than the range defined because
	#it includes dummy return instructions for the appcode entry points
	load_size = highest_addr - 2 
	if load_size % proc.row_size != 0:
		raise ValueError("Payload should be a multiple of the flash row size for the chip.")

	print "Payload Size: %d" % load_size

	return {'load_size': load_size}



class LogStatement:
	def pull_info(self, log):
		self.address = log.current_address

	def keep(self):
		"""
		Override in subclasses if this logging statement doesn't stand on its own but 
		just sets state for future logging statements.  It will be processed and then
		deleted from the Log object list of logging statements
		"""
		return True

	def error(self):
		"""
		Override for logging statements that indicate errors, rather than ones that are just informational
		"""
		return False

	def format_line(self, symtab, use_colors):
		HEADER = '\033[95m'
		OKBLUE = '\033[94m'
		OKGREEN = '\033[92m'
		WARNING = '\033[93m'
		FAIL = '\033[91m'
		ENDC = '\033[0m'

		func = None

		color = OKBLUE
		if self.error():
			color = FAIL

		if use_colors is not True:
			HEADER = ''
			color = ''
			ENDC = ''

		if symtab is not None:
			func = symtab.map_address(self.address)

		header = 'Address 0x%X: ' % self.address
		if func is not None:
			header = HEADER + '%s+%d: ' % (func[0], func[1]) + ENDC

		return header + color + self.format() + ENDC


import unittest
import os.path
import os
from nose.tools import *
from pymomo.exceptions import *
from pymomo.pcb import CircuitBoard
import tempfile
import shutil

def test_check_attributes():
	#Board is missing Company and Dimension unit attributes
	board = os.path.join(os.path.dirname(__file__), 'eagle', 'controller_missing_attrs.brd')
	b = CircuitBoard(board)

	assert b.is_clean() == False
	errors = b.get_errors()
	warnings = b.get_warnings()

	print errors
	print warnings

	assert len(errors) == 3
	assert len(warnings) == 0



#proxy12.py
#Proxy object for all modules that have a mib12 executive

import proxy
from pymomo.utilities.typedargs.annotate import returns, param, annotated
from collections import namedtuple
from pymomo.commander.exceptions import *

#Printer functions for displaying return values.
def print_status(status):
	"""
	Break out executive status bits and print them.
	"""

	print "Executive Status Register"
	print "Serial Number: %d" % status.serial
	print "HW Type: %d" % status.hwtype
	print "First Application Row: %d" % status.approw
	print "Runtime Status: 0x%X" % status.status

	if status.trapped:
		print "\n***Module has crashed and is waiting for debugging; trap bit is set.***\n"
	else:
		print "\nModule is running normally.\n"

class MIB12ProxyObject (proxy.MIBProxyObject):
	"""
	Proxy object for all 8-bit PIC modules that run the pic12_executive.
	Executive functionality is implemented here.
	"""

	@returns(desc='application firmware checksum', data=True)
	def checksum(self):
		"""
		Get the 8-bit application checksum.
		"""
		return self.rpc(1,2, result_type=(1,False))['ints'][0]

	@returns(desc='module status register', data=True, printer=print_status)
	def status(self):
		"""
		Get the module status register.

		Returns executive version, runtime parameters, hw type, executive size and whether the module has crashed.
		"""

		res = self.rpc(1,4, result_type=(2,False))
		status = namedtuple("ExecutiveStatus", ['serial', 'hwtype', 'approw', 'status', 'trapped'])

		status.serial = res['ints'][0] & 0xFF
		status.hwtype = res['ints'][0] >> 8
		status.approw = res['ints'][1] & 0xFF
		status.status = res['ints'][1] >> 8
		status.trapped = bool(status.status & 1<<7) 

		return status

	@param('location','integer','positive',desc='RAM address to read')
	@param('type', 'string', ('list', ['uint8']), desc='Type of variable to read (supports: uint8)')
	@returns(desc='variable contents', data=True)
	def readram(self, location, type='uint8'):
		res = self.rpc(1,3, location, result_type=(0,True))
		return ord(res['buffer'][0])

	@annotated
	def reset(self):
		"""
		Reset the application module.
		"""

		try:
			self.rpc(1, 1)
		except RPCException as e:
			if e.type != 7:
				raise e 


#Caveats and possible issues
#Mac OS X
# - when using a virtualenv, readline is not properly installed into the virtualenv
#   and cannot be imported.  You need to install it using easy_install as described here
#   http://calvinx.com/tag/readline/
from setuptools import setup, find_packages

import os
from os.path import basename

import re
def parse_version():
    VERSIONFILE="pymomo/__version__.py"
    verstrline = open(VERSIONFILE, "rt").read()
    VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
    mo = re.search(VSRE, verstrline, re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

def list_data_files():
    result = [ ]
    dirname = os.path.join(os.path.dirname(__file__), 'pymomo')
    print dirname
    for root, dirs, files in os.walk(os.path.join(dirname, 'config')):
        for filename in files:
            result.append( os.path.join( root, filename )[len(dirname)+1:] )
    return result

setup(
    name = "pymomo",
    packages = find_packages(),
    version = parse_version(),
    license = "LGPLv3",
    install_requires=[
        "beautifulsoup4==4.3.2",
        "Cheetah==2.4.4",
        "cmdln==1.1.2",
        "colorama==0.3.3",
        "decorator==3.4.0",
        "intelhex==1.5",
        "Markdown==2.5.2",
        "nose==1.3.4",
        "py==1.4.26",
        "pycparser==2.10",
        "pyparsing==2.0.3",
        "pyserial==2.7",
        "pytest==2.6.4",
        "six==1.9.0",
        "xlsxwriter>=0.6.7",
        "pint>=0.6"
    ],
    package_data={ #This could be better
        'pymomo': list_data_files()
    },
    entry_points={
        'console_scripts': [
            'momo = pymomo.scripts.momo:main',
            'momo-mod = pymomo.scripts.modtool:main',
            'momo-gsm = pymomo.scripts.gsmtool:main',
            'momo-pcb = pymomo.scripts.pcbtool:main',
            'momo-hex = pymomo.scripts.hextool:main',
            'momo-mib = pymomo.scripts.mibtool:main',
            'momo-reportinator = pymomo.scripts.reportinator:main',
            'momo-picunit = pymomo.scripts.picunit:main',
            'momo-multisensor = pymomo.scripts.multisensor:main',
            'momo-sensor = pymomo.scripts.momosensor:main'
        ]
    },
    description = "WellDone Mobile Monitor (MoMo) Interaction Library",
    author = "WellDone International",
    author_email = "info@welldone.org",
    url = "http://github.com/welldone/pymomo",
    keywords = ["momo", "remote", "monitoring", "embedded", "hardware"],
    classifiers = [
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules"
        ],
    long_description = """\
PyMoMo
------

A python library for interacting with MoMo devices.  See https://momo.welldone.org.

"""
)


#Routines for building part objects from eagle board files
from pint import UnitRegistry
from pymomo.exceptions import *
from pymomo.utilities.typedargs import type_system

from ..package import Pad, Pin, Package, Placement

ureg = UnitRegistry()

def part_from_board_element(elem, variant, packages):
	"""
	Create a Part object from an element in an EAGLE board file. 

	Automatically extract all of the known attributes of the part to fill in the
	reference identifier, package information, number of pads/pins, footprint,
	description, etc.

	packages should be a list of XML elements corresponding to the various lists of packages 
	for this board.

	Returns a tuple of Part,Boolean
	The Part describes this part and may be None if it is not populated in this variant
	The Boolean is true is this part is valid and populated in this variant and false otherwise.
	"""

	pkg = elem.get('package', None)

	name = elem.get('name', 'No Name')
	value = elem.get('value', 'No Value')

	value_override = find_attribute(elem, 'VALUE', variant)
	if value_override is not None:
		value = value_override

	#allow overriding this package with a custom name
	pkgname = find_attribute(elem, 'FOOTPRINT', variant, pkg)
	if pkg is not None:
		pkg_info = find_package(packages, pkg, display_name=pkgname)
	else:
		pkg_info = Package(str(pkgname), [], [])

	desc = find_attribute(elem, 'DESCRIPTION', variant, None)
	pop_attr = find_attribute(elem, 'POPULATE', variant, "yes")
	
	if pop_attr.lower() == "no":
		return None, True
	elif pop_attr != "yes":
		raise ValidationError("Unknown value in POPULATE attribute in element %s: %s" % (name, pop_attr))

	manu, mpn = find_mpn(elem, variant)
	dist, distpn = find_distpn(elem, variant)

	#Get physical location information
	x = float(elem.get('x')) * ureg.millimeter
	y = float(elem.get('y')) * ureg.millimeter

	rot = elem.get('rot')
	if rot is None:
		rot = 0.0
	else:
		rot = float(elem.get('rot')[1:])			#Rotation is specified at R## where ## is the angle in degrees.

	placement = Placement(x, y, rot)

	if (mpn is not None and manu is not None) or (dist is not None and distpn is not None):
		args = {}
		args['name'] = name
		args['package'] = pkg_info
		args['placement'] = placement
		args['dist'] = dist
		args['distpn'] = distpn
		args['mpn'] = mpn
		args['manu'] = manu
		args['value'] = value
		args['desc'] = desc

		return type_system.convert_to_type(args, 'logical_part'), True

	return None, False

def attrib_name(name, variant):
	if variant == "MAIN" or variant == "":
		return name

	return name + '-' + variant

def find_attribute(part, name, variant, default=None):
	"""
	Find the attribute corresponding to the given variant, or if that doesn't exist, 
	the value corresponding to MAIN, otherwise return the given default
	"""

	attrib_var = part.find("./attribute[@name='%s']" % attrib_name(name, variant))
	attrib_main = part.find("./attribute[@name='%s']" % attrib_name(name, 'MAIN'))

	if attrib_var is not None:
		return attrib_var.get('value')

	if attrib_main is not None:
		return attrib_main.get('value')

	return default

def find_mpn(part, variant):
	"""
	See if this part has the variables MPN[-variant] and MANU[-variant] defined
	return None if either is undefined.  Otherwise return the tuple (manu, mpn)
	"""

	mpn  = find_attribute(part, 'MPN', variant)
	manu = find_attribute(part, 'MANU', variant)

	if mpn is None or manu is None:
		return (None, None)

	return (manu, mpn)

def find_distpn(part, variant):
	"""
	Try to find a distributor part number for this part

	This can be specified either using a single combined attribute for Digikey
	parts (DIGIKEY-PN) or by using 2 attributes specifying the distributor and
	the part number (DIST and DIST-PN).  If any of these are blank, they are returned
	as None.
	"""

	#Look for a digikey pn attribute as a shortcut for digikey parts
	pn_elem = find_attribute(part, 'DIGIKEY-PN', variant)
	if pn_elem is not None:
		return ("Digikey", pn_elem)

	pn_elem = find_attribute(part, 'DIST-PN', variant)
	dist_elem = find_attribute(part, 'DIST', variant)

	dist = None
	pn = None

	if pn_elem is not None:
		pn = pn_elem
	if dist_elem is not None:
		dist = dist_elem

	return (dist, pn)

def find_package(packages, find_name, display_name):
	"""
	Create a Package object from the named package
	"""

	found = False

	for packagelist in packages:
		package = packagelist.find("./package[@name='%s']" % str(find_name))
		if package is not None:
			found = True
			break

	if found is False:
		raise ValidationError("Invalid board file, a package was used but not defined", name=find_name)

	eagle_pads = package.findall("./smd")
	eagle_pins = package.findall("./pad")

	pads = []
	pins = []

	#Eagle boards store all coordinates in mm regardless of how it is displayed
	for epad in eagle_pads:
		x = float(epad.get('x'))  * ureg.millimeter
		y = float(epad.get('y'))  * ureg.millimeter
		w = float(epad.get('dx')) * ureg.millimeter
		h = float(epad.get('dy')) * ureg.millimeter

		pad = Pad(x, y, w, h)
		pads.append(pad)

	for epin in eagle_pins:
		x = float(epin.get('x'))  * ureg.millimeter
		y = float(epin.get('y'))  * ureg.millimeter
		drill = float(epin.get('drill'))  * ureg.millimeter

		pin = Pin(x,y, drill)
		pins.append(pin)

	return Package(display_name, pins, pads)



#build.py
#Return the build settings json file.

import json as json
from paths import MomoPaths
import os.path
from pymomo.utilities import deprecated
import sys
from pymomo.utilities.typedargs.annotate import *
from pymomo.exceptions import *
from collections import namedtuple
from copy import deepcopy
import itertools
import os
from paths import MomoPaths

@takes_cmdline
def build(args):
	"""
	Invoke the scons build system from the current directory, exactly as if 
	the scons tool had been invoked. 
	"""

	import pymomo.utilities.invoke
	import SCons.Script

	paths = MomoPaths()

	site_path = os.path.abspath(paths.site_tools)

	all_args = ['momo', '--site-dir=%s' % site_path]
	sys.argv = all_args + list(args)
	SCons.Script.main()

def load_settings(filename=None):
	local_file = True

	if filename is None:
		paths = MomoPaths()
		filename = os.path.join(paths.config,'build_settings.json')
		local_file = False

	try:
		with open(filename,'r') as f:
			return json.load(f)
	except IOError:
		if local_file:
			raise ArgumentError("Could not open settings file '%s'" % str(filename))
		else:
			raise APIError('Could not load global build settings file (config/build_settings.json)')

def load_chip_info(chip):
	"""
	Load chip info from chip_settings dictionary in build_settings.json
	"""
	conf = load_settings()
	settings = conf['mib12']['chip_settings'][chip]

	aliases = []
	if 'aliases' in settings:
		aliases = settings['aliases']

	default = deepcopy(conf['mib12']['default_settings'])
	chip_info = merge_dicts(default, settings)

	return (aliases, chip_info)

def merge_dicts(a, b):
    "merges b into a"

    for key in b:
        if key in a:
            if isinstance(a[key], dict) and isinstance(b[key], dict):
                merge_dicts(a[key], b[key])
            else:
            	a[key] = b[key]
        else:
            a[key] = b[key]
    
    return a

MISSING = object()

ModuleSettings = namedtuple('ModuleSettings', ['overlays', 'settings'])

class TargetSettings:
	"""
	A class that contains a dictionary of all the settings defined for
	a given chip in a ChipFamily.  The settings are a combination of
	default settings for the family which can be overwritten by chip
	specific settings.
	"""

	def __init__(self, name, settings, family):
		self.name = name
		self.settings = settings
		self.family = family

	def output_name(self):
		return self.name.replace('/', '_').replace(":","_")

	def arch_name(self):
		mod,arch = self.name.split(':')
		return arch.replace('/',"_")

	def arch_list(self):
		return self.name.split(':')[1]

	def archs(self, as_list=False):
		"""
		Return a set containing all of the architectures used in this TargetSettings
		object.
		"""

		archs = self.arch_list().split('/')
		
		if as_list:
			return archs

		return set(archs)

	def module_name(self):
		"""
		Return the module name used to produce this TargetSettings object
		"""

		name = self.name.split(':')[0]
		return name

	def retarget(self, remove=[], add=[]):
		"""
		Return a TargetSettings object for the same module but with some of the architectures
		removed and others added.
		"""

		archs = self.arch_list().split('/')

		for r in remove:
			if r in archs:
				archs.remove(r)

		archs.extend(add)

		archstr = "/".join(archs)
		return self.family.find(archstr, self.module_name())

	def build_dirs(self):
		"""
		Return the build directory hierarchy:
		Defines:
		- build: build/chip
		- output: build/output
		- test: build/test/chip
		where chip is the cannonical name for the chip passed in
		"""

		arch = self.arch_name()

		build = os.path.join('build', arch)
		output = os.path.join('build', 'output')
		test = os.path.join('build', 'test', arch)

		return {'build': build, 'output': output, 'test': test}

	def property(self, name, default=MISSING):
		"""
		Get the value of the given property for this chip, using the default
		value if not found and one is provided.  If not found and default is None,
		raise an Exception.
		"""

		if name in self.settings:
			return self.settings[name]	
		
		if default is not MISSING:
			return default

		raise ArgumentError("property %s not found for target '%s' and no default given" % (name, self.name))

	def includes(self):
		"""
		Return all of the include directories for this chip as a list.
		"""

		paths = MomoPaths()
		base = paths.modules
		
		incs = [y for x,y in self.settings.iteritems() if x.endswith('includes')]
		includes = itertools.chain(*incs)

		fullpaths = [os.path.normpath(os.path.join(base, x)) for x in includes]
		fullpaths.append(os.path.normpath(os.path.abspath(self.build_dirs()['build'])))

		return fullpaths

	def extra_sources(self):
		"""
		If the architectures have specified that extra source files be included, return a list of paths to
		those source files.
		"""

		paths = MomoPaths()
		base = paths.modules
		
		srcs = [y for x,y in self.settings.iteritems() if x.endswith('sources')]
		sources = itertools.chain(*srcs)

		fullpaths = [os.path.normpath(os.path.join(base, x)) for x in sources]
		return fullpaths

	def arch_prefixes(self):
		"""
		Return the initial 1, 2, ..., N architectures as a prefix list

		For arch1/arch2/arch3, this returns
		[arch1],[arch1/arch2],[arch1/arch2/arch3]
		"""

		archs = self.archs(as_list=True)
		prefixes = []

		for i in xrange(1, len(archs)+1):
			prefixes.append(archs[:i])

		return prefixes


class ChipFamily:
	"""
	Given a family of chips that may be targeted together from the same source 
	code, load in the chip names and aliases and store it in a standardized
	fashion.  The data is loaded from config/build_settings.json.
	"""

	def __init__(self, surname, localfile=None, modulefile=None):
		"""
		Build a ChipFamily from the family name, e.g. mib12, mib24
		If localfile is not None, use that file to load the settings from,
		otherwise load settings from gobal build_settings file
		"""

		settings = load_settings(localfile)
		if not surname in settings:
			raise InternalError("Could not find family %s in config file" % surname)

		family = settings[surname]
		if "modules" not in family:
			family['modules'] = {}
		if "module_targets" not in family:
			family['module_targets'] = {}
		if "architectures" not in family:
			family['architectures'] = {}

		#Add in all information from local modules file if there is one
		if modulefile is not None:
			modsettings = load_settings(modulefile)

			if "modules" in modsettings:
				merge_dicts(family['modules'], modsettings['modules'].copy())
			if "module_targets" in modsettings:
				merge_dicts(family['module_targets'], modsettings['module_targets'].copy())
			if "architectures" in modsettings:
				merge_dicts(family['architectures'], modsettings['architectures'].copy())

		self.archs = {}
		self.module_targets = {}
		self.modules = {}
		self.default_settings = {}

		self._load_architectures(family)
		self._load_module_targets(family)
		self._load_modules(family)

		if "default_settings" in family:
			self.default_settings = family['default_settings']


	def find(self, name, module=None):
		"""
		Given a target name and optionally a module name, return a settings object
		for that target 
		"""
		return self._load_target(name, module)

	def targets(self, module):
		"""
		Return a sequence of all of the targets for the specified module.
		Modules that have no entry in module_targets in build_setings.json
		target all family defined targets by default.
		"""

		if module in self.module_targets:
			return [self.find(x, module) for x in self.module_targets[module]]

		return [self.find(x, module) for x in self.known_targets.keys()]

	def for_all_targets(self, module, func, filter=None):
		"""
		Call func once for all of the targets of this module 
		"""
		
		for target in self.targets(module):
			if filter is None or filter(target) == True:
				func(target)

	def validate_target(self, target):
		"""
		Make sure that the specified target only contains architectures that we know about. 
		"""

		archs = target.split('/')

		for arch in archs:
			if not arch in self.archs:
				return False

		return True

	def _load_target(self, target, module=None):
		"""
		Given a string specifying a series of architectural overlays as:
		<arch 1>/<arch 2>/... and optionally a module name to pull in
		module specific settings, return a TargetSettings object that
		encapsulates all of the settings for this target.
		"""

		mod = ModuleSettings({}, {})
		if not self.validate_target(target):
			raise ArgumentError("Target %s is invalid, check to make sure every architecture in it is defined" % target)

		if module is not None:
			if module not in self.modules:
				raise ArgumentError("Unknown module name passed: %s" % module)

			mod = self.modules[module]

		settings = deepcopy(self.default_settings)
		archs = target.split('/')

		for arch in archs:
			arch_settings = deepcopy(self.archs[arch])

			if arch in mod.overlays:
				arch_settings = merge_dicts(arch_settings, mod.overlays[arch])

			#Allow this architecture to overlay previous architectures as well
			if "overlays" in arch_settings:
				for arch2 in archs:
					if arch2 == arch:
						break

					if arch2 in arch_settings["overlays"]:
						arch_settings = merge_dicts(arch_settings, arch_settings["overlays"][arch2])

				del arch_settings["overlays"]

			settings = merge_dicts(settings, arch_settings)

		settings = merge_dicts(settings, mod.settings)

		targetname = "%s:%s" % (str(module), target)

		return TargetSettings(targetname, settings, self)

	def _load_module_targets(self, family):
		if "module_targets" in family:
			for mod, targets in family['module_targets'].iteritems():
				for target in targets:
					if not self.validate_target(target):
						raise InternalError("Module %s targets unknown architectures '%s'" % (mod, target))

				self.module_targets[mod] = targets

	def _load_architectures(self, family):
		"""
		Load in all of the architectural overlays for this family.  An architecture adds configuration
		information that is used to build a common set of source code for a particular hardware and sitation.
		They are stackable so that you can specify a chip and a configuration for that chip, for example.
		"""

		if "architectures" not in family:
			raise InternalError("required architectures key not in build_settings.json for desired family")


		for key, val in family['architectures'].iteritems():
			if not isinstance(val, dict):
				raise InternalError("All entries under chip_settings must be dictionaries")

			self.archs[key] = deepcopy(val)

	def _load_modules(self, family):
		if "modules" not in family:
			raise InternalError("required modules key not in build_settings.json for desired family")

		for modname in family['modules']:
			overlays = {}
			rawmod = family['modules'][modname]

			if 'overlays' in rawmod:
				overlays = rawmod['overlays']
				del rawmod['overlays']

			mod = ModuleSettings(overlays, rawmod)
			self.modules[modname] = mod




import os.path
import sys
import subprocess
import functools
import platform
import os

from pkg_resources import resource_filename, Requirement

class MissingConfigError(Exception):
  """A required configuration environment variable was not found."""
  def __init__(self, var):
    self.msg = "A required configuration environment variable (%s) was not found" % var
  def __str__(self):
  	return self.msg

class MomoPaths:
	def __init__(self):
		self.config = resource_filename(Requirement.parse("pymomo"), "pymomo/config")
		self.base = os.environ.get('MOMOPATH')

		if self.base == None:
			raise MissingConfigError('MOMOPATH')

		self.modules = os.path.join(self.base, 'momo_modules')
		self.templates = os.path.join(self.config, 'templates')
		self.pcb = os.path.join(self.base, 'pcb')
		self.site_tools = os.path.join(self.config, 'site_scons')
		self.settings = self._find_settings_dir()

	def _find_settings_dir(self):
		"""
		Find a per user settings directory that is appropriate for each
		type of system that we are installed on.
		"""

		system = platform.system()

		basedir = None

		if system == 'Windows':
			if 'APPDATA' in os.environ:
				basedir = os.environ['APPDATA']
		elif system == 'Darwin':
			basedir = os.path.expanduser('~/Library/Preferences')

		#If we're not on Windows or Mac OS X, assume we're on some
		#kind of posix system where the appropriate place would be
		#~/.config
		if basedir is None:
			basedir = os.path.expanduser('~')
			basedir = os.path.join(basedir, '.config')

		settings_dir = os.path.abspath(os.path.join(basedir, 'WellDone-MoMo'))
		if not os.path.exists(settings_dir):
			os.makedirs(settings_dir, 0755)

		return settings_dir

	def select(self, *args, **kwargs):
		"""
		Given a base path, return a list of all of the files under that path that match the given filter.
		*args should be a list of path components that are joined together using path.join to form the base
		search path

		**kwargs takes the following options:
		- filter: 	a function that is applied to the file paths found using this function to select only a subset
					filter is passed 3 arguments, (the absolute file path, the file name, its extension)
		- recursive: a boolean.  If True or not specified, search under all subdirectories as well
		- dirs: a boolean.  If True, only search directory names, if False or unspecified, search only file names
		"""

		base = os.path.join(*args)
		files = []

		for (dirpath, dirnames, filenames) in os.walk(base):
			if 'dirs' in kwargs and kwargs['dirs'] == True:
				entries = dirnames
			else:
				entries = filenames

			full_paths = map(lambda x: os.path.join(base, dirpath, x), entries)


			if "filter" not in kwargs:
				files.extend(full_paths)
			else:
				split_names = map(lambda x: os.path.splitext(x), entries)
				wanted_files = [full_paths[i] for i in xrange(0, len(entries)) if kwargs['filter'](full_paths[i], split_names[i][0], split_names[i][1]) == True]

				files.extend(wanted_files)

			if 'recursive' in kwargs and kwargs['recursive'] == False:
				break

		return files

def memoize(obj):
	cache = obj.cache = {}

	@functools.wraps(obj)
	def memoizer(*args, **kwargs):
		key = str(args) + str(kwargs)
		if key not in cache:
			cache[key] = obj(*args, **kwargs)
		return cache[key]
	
	return memoizer

@memoize
def convert_path(path):
	"""
	If we are running on cygwin and passing an absolute path to a utility that is not cygwin
	aware, we need to convert that path to a windows style path. 
	"""

	if not os.path.isabs(path):
		return '"' + path + '"'

	if sys.platform == 'cygwin':
		out = subprocess.check_output(['cygpath', '-mw', path])
		out = out.lstrip().rstrip()
		return '"' + out + '"'

	return '"' + path + '"'



from pymomo.commander.exceptions import *
from datetime import datetime
import struct

class SensorEvent:
	"""
	An event logged by a sensor module
	"""

	def __init__(self, buffer):
		if len(buffer) != 12:
			raise TypeException( 'SensorEvent', 'Length should have been 12, was %d' % len(buffer) )

		stream, metadata, timestamp_year, timestamp_month, timestamp_day, timestamp_hours, timestamp_minutes, timestamp_seconds, value = struct.unpack('BBBBBBBBI', buffer)

		self.stream = stream
		self.metadata = metadata
		self.timestamp = (timestamp_year, timestamp_month , timestamp_day, timestamp_hours, timestamp_minutes, timestamp_seconds)
		self.value = value


# simulator.py
# Generic interface for controlling PIC microchip simulators
#
# Designed to work with sim30 from Microchip for simulating PIC24
# chips and gpsim to simulate pic12 and pic16 chips.

import simulators
from pymomo.utilities.typedargs.annotate import *
from pymomo.exceptions import *
from multiprocessing import Process, Queue, Pipe
import atexit
import os.path
import os
import tempfile

def _spawn_simulator(sim_type, child_pipe):
	"""
	Spawn a simulator of the given type, attaching child_pipe to it
	so that we can communicate with it.
	"""

	sim = sim_type(child_pipe)
	sim.run()

def _quit_simulator(sim):
	"""
	Quit the simulator process.
	"""
	sim.exit()

@context("Simulator")
class Simulator:
	"""
	Generic interface for simulating embedded microprocessors.
	Subclasses should implement specific methods as a worker
	object that is spawned in a separate process.
	"""

	@param("type", "string", desc="Type of simulator to create (pic24 or pic12)")
	def __init__(self, type):
		"""
		Create a simulator object in a separate process 
		"""

		if not hasattr(simulators, type):
			raise ArgumentError("unknown simulator type '%s'" % str(type))

		self.sim_type = getattr(simulators, type)
		self._pipe, child_pipe = Pipe(duplex=True)

		self.process = Process(target=_spawn_simulator, args=(self.sim_type, child_pipe))
		self.process.start()

		#Make a list of any temporary files generated by this object for deletion
		self.temporaries = []

		#Make sure we always quit the simulator when we exit so that we don't hang the
		#executing proces
		atexit.register(_quit_simulator, self)

	@finalizer
	def exit(self):
		"""
		Tell the simulator to quit and wait for it to exit
		"""
		
		self.finish()

		#Clean up any temporary files
		for i in self.temporaries:
			os.remove(i)

		#If we are called twice, make sure we don't try to clean up again.
		self.temporaries = []

	@annotated
	def finish(self):
		"""
		Close the underlying simulator process, guaranteeing that any files that output
		during the simulation are created and valid after this call returns.
		"""

		if self.process.is_alive():
			self._command("quit")
			self.process.join()

	@param("program", "path", "readable", desc="Path of program object file to load")
	def load(self, program):
		"""
		Load the program object file into the simulator
		"""

		# Try to determine file type
		type = 'unknown'
		ext = os.path.splitext(program)[1]
		if ext == '.coff' or ext == '.cof':
			type = 'coff'
		elif ext == '.elf':
			type = 'elf'
		elif ext == '.hex':
			type = 'hex'

		self._command('load_program', program=program, type=type)

	@param("name", "string", desc="Name of the simulator parameter to set")
	@param("value", "string", desc="Value of the parameter")
	def set_param(self, name, value):
		"""
		Set a simulator specific parameter by name.
		"""

		self._command('set_param', name=name, value=value)

	@annotated
	def attach_log(self):
		"""
		Upon completion of this simulation, generate a log file containing
		the output of the simulated code.  The log file is not guaranteed to be generated
		until finish() is called on the simulator.
		"""

		path = self._build_temp()
		self._command('set_log', file=path)
		self.logfile = path

	@returns(desc="log contents")
	def get_log(self):
		"""
		Return the log attached to the simulator in a simulator-dependent manner as a string.
		This method is only guaranteed to return all of the log's contents if finish() has
		been called before the call to get_log.  This is because the simulator subprocesses
		may not flush data to the log file until they are closed and we cannot use a pipe because
		not all of the external simulator programs support it.
		"""

		if not hasattr(self, 'logfile'):
			raise InternalError('You must attach a log using attach_log before calling get_log')

		try:
			with open(self.logfile, 'r') as f:
				contents = f.read()
		except IOError as e:
			raise InternalError('Log file could not be read because: %s' % str(e))

		return contents

	@param("command", "string", desc="Raw command string to execute in simulator")
	def raw_command(self, command):
		"""
		Pass a raw string command to the underlying simulator and execute it.  This
		can be useful if there is not a high level API function for achieving what
		you need to accomplish.
		"""

		self._command('raw_command', command=command)

	@returns(desc="reason")
	@param("wait", "integer", desc="maximum number of seconds to wait (-1 is forever)")
	def wait(self, wait):
		"""
		Wait for the simulator to finish executing code and return to being able
		to process commands.
		"""

		return self._command('wait', timeout=wait)

	@returns(desc="ready for commands")
	def ready(self):
		"""
		Return if the simulator is ready to accept new commands or if it is running
		code.
		"""

		return self._command('ready')

	@annotated
	def execute(self):
		"""
		Begin or continue program execution
		"""

		self._command('execute')

	#Communication functions to communicate with the simulator worker objects
	def _send_cmd(self, method, **kwargs):
		msg = {'method': method}
		msg.update(kwargs)
		self._pipe.send(msg)

	def _receive_response(self, timeout=3):
		if not self._pipe.poll(timeout):
			self.process.terminate()
			raise TimeoutError("Waiting for a response from child simulator process.")

		response = self._pipe.recv()
		return response

	def _command(self, method, **kwargs):
		self._send_cmd(method, **kwargs)

		if "timeout" in kwargs:
			timeout = kwargs['timeout']+1
			if kwargs['timeout'] < 0:
				timeout = None

			resp = self._receive_response(timeout=timeout)
		else:
			resp = self._receive_response()

		if resp['error'] is True:
			params = {x:y for x,y in resp.iteritems() if x != 'message' and x != 'error'}
			raise InternalError(resp['message'], **params)

		return resp.get('result', None)

	def _build_temp(self):
		"""
		Build a temporary file with no spaces in the name for use with simulator subprocesses.
		"""

		outfile = tempfile.NamedTemporaryFile(delete=False)
		outfile.close()

		if " " in outfile.name:
			os.remove(outfile.name)
			raise InternalError("Temporary direction has space in file name, sim30 cannot handle this, check the TMPDIR,TEMP and TMP environment variables.  Path was '%s'" % outfile)
		

		self.temporaries.append(outfile.name)
		return outfile.name



from logentry import RawLogEntry, MessageEntry
from descriptor import LogDefinitionMap
from pymomo.utilities.typedargs.annotate import *
from pymomo.utilities import typedargs

class LogEntry:
	"""
	An object representing a complete log entry 
	"""

	def __init__(self, msg):
		self.params = []
		self.header = msg

@context()
class SystemLog:
	"""
	A list of log entries from the MoMo system log

	The MoMo controller keeps an ordered list of timestamped
	logged events, which can contain arbitrary parameters. 
	This class encapsulates one such list and allows you to
	access, filter and display the log entries.
	"""

	@annotated
	def __init__(self, raw_entries):
		"""
		Go through all of the entries and merge continuations
		"""
		self.entries = []

		entries = []
		self._mapper = LogDefinitionMap()

		for entry in raw_entries:
			if len(entries) != 0 and entry.typecode == RawLogEntry.ContinuationType:
				entries[-1].merge(entry)
			else:
				entries.append(entry)

		self._combine_entries(entries)

	def _combine_entries(self, entries):
		"""
		Combine entries into logical logging events including parameters
		"""
		current = None

		for entry in entries:
			if entry.typecode == RawLogEntry.NewLogEntryType:
				if current is not None:
					self.entries.append(current)

				current = LogEntry(MessageEntry(entry.data))
			elif current is not None:
				if entry.typecode == RawLogEntry.IntegerListType:
					ints = self._parse_ints(entry)
					current.params.extend(ints)
				elif entry.typecode == RawLogEntry.StringType:
					current.params.append(('array', entry.data[:-1])) #Chomp extra '\0' at the end of string entries
				else:
					current.params.append(('array', entry.data))

		if current is not None:
			self.entries.append(current)

	def _format_params(self, entry, ldf):
		"""
		Attempt to format the parameters in this log entry.
		"""
		fmtd = None

		if ldf.is_valid_length(entry.params):
			fmtd = []

			try:
				typed_params = ldf.assign_types(entry.params)

				for desc, data in typed_params:
					s = typedargs.format_value(data[1], desc.type, desc.format)
					fmt = "%s: %s" % (desc.name, s)
					fmtd.append(fmt)
			except MoMoException:
				fmtd = None

		#If we were not successful in formatting the parameters, just print them
		if fmtd is None:
			fmtd = [str(x[1]) for x in entry.params]

		return fmtd

	def _parse_ints(self, entry):
		if entry.length % 2 != 0:
			raise ValidationError("Integerlist whose length was not a multiple of two found", entry=entry, length=entry.length)

		ints = []

		for i in xrange(0, entry.length, 2):
			low = ord(entry.data[i])
			high = ord(entry.data[i+1])

			val = ((high & 0xFF) << 8) | low
			ints.append(('integer', val))

		return ints

	@param("file", "path", "readable", desc='Path of an ldf file to use to parse this system log')
	def add_ldf(self, file):
		"""
		Add an ldf file to help parse log entries
		"""

		self._mapper.add_ldf(file)

	def __str__(self):
		val = ""

		for entry in self.entries:
			ldf = self._mapper.map(entry.header.msg)
			if ldf.message is not None:
				desc = ldf.message
			else:
				desc = ldf.name

			line = "%s (%s) %s\n" % (entry.header.stream_name(), entry.header.timestamp, desc)
			val += line

			#Format and print the parameters
			params = self._format_params(entry, ldf)
			for param in params:
				ind = " - "
				ind += param + '\n'
				val += ind

		return val.rstrip()

	@annotated
	def display(self):
		val = str(self)
		if val != "":
			print str(self)


#api.py
#Routines for dumping the MIB API region of a mib12 executive module and verifying
#the contents to make sure they have not been stomped on by some other process.

from pymomo.hex8.decode import *
from pymomo.utilities.paths import MomoPaths
from pymomo.utilities import build
from config12 import MIB12Processor
import intelhex

class MIBAPI:
	def __init__(self, hexfile, chip):
		with open(hexfile, "r"):
			self.hf = intelhex.IntelHex16bit(hexfile)

		proc = MIB12Processor.FromChip(chip)
		self.api_base = proc.api_range[0]

		self.valid = self.verify_api()

	def verify_api(self):
		"""
		Verify that all instructions in the MIB api region are either retlw 0
		or goto.
		"""
		for i in xrange(0, 16):
			try:
				val = decode_retlw(self.hf, self.api_base + i)
				if val == 0:
					continue

				return False
			except:
				pass

			try: 
				decode_goto(self.hf, self.api_base + i)
				continue
			except:
				pass

			return False

		return True

	def print_api(self):
		print "MIB API Block"
		print "Valid:", self.valid
		print "\nTable Contents Follow"

		for i in xrange(0, 16):
			try:
				val = decode_retlw(self.hf, self.api_base + i)
				print "%d: retlw 0x%x" % (i, val)
				continue
			except:
				pass

			try: 
				addr = decode_goto(self.hf, self.api_base + i)
				print "%d: goto 0x%x" % (i, addr)
				continue
			except:
				pass

			print "%d: Invalid Instruction (0x%x)" % (i, self.hf[self.api_base + i])



import sys
import os
import shlex

from pymomo.utilities.typedargs.shell import HierarchicalShell, posix_lex
from pymomo.exceptions import *
from pymomo.utilities.typedargs import annotate
from pymomo.commander.meta import initialization
from pymomo.utilities import build
from pymomo.utilities.rcfile import RCFile
from multiprocessing import freeze_support
import traceback

def main():
	line = sys.argv[1:]

	norc=False
	if len(line) > 0 and line[0] == '--norc':
		norc = True
		line = line[1:]

	if len(line) > 0 and line[0] == '--rcfile':
		rc = RCFile('momo')
		print rc.path
		return 0

	shell = HierarchicalShell('momo', no_rc=norc)
	
	shell.root_update(annotate.find_all(initialization))
	
	shell.root_add("build", "pymomo.utilities.build,build")
	shell.root_add("SystemLog", "pymomo.syslog")
	shell.root_add("pcb", "pymomo.pcb")
	shell.root_add('ControllerBlock', "pymomo.hex,ControllerBlock")
	shell.root_add('HexFile', "pymomo.hex,HexFile")
	shell.root_add('Simulator', "pymomo.sim,Simulator")

	finished = False

	try:
		while len(line) > 0:
			line, finished = shell.invoke(line)
	except APIError:
		traceback.print_exc()
		return 1
	except MoMoException as e:
		print e.format()
		#if the command passed on the command line fails, then we should
		#just exit rather than drop the user into a shell.
		return 1

	#Setup file path and function name completion
	#Handle special case of importing pyreadline on Windows
	#See: http://stackoverflow.com/questions/6024952/readline-functionality-on-windows-with-python-2-7
	import glob
	try:
		import readline
	except ImportError:
		import pyreadline as readline

	def complete(text, state):
		buf = readline.get_line_buffer()
		if buf.startswith('help ') or " " not in buf:
			funcs = shell.valid_identifiers()
			return filter(lambda x: x.startswith(text), funcs)[state]

		return (glob.glob(os.path.expanduser(text)+'*')+[None])[state]

	readline.set_completer_delims(' \t\n;')
	#Handle Mac OS X special libedit based readline
	#See: http://stackoverflow.com/questions/7116038/python-tab-completion-mac-osx-10-7-lion
	if readline.__doc__ is not None and 'libedit' in readline.__doc__:
		readline.parse_and_bind("bind ^I rl_complete")
	else:
		readline.parse_and_bind("tab: complete")

	readline.set_completer(complete)

	#If we ended the initial command with a context, start a shell
	if not finished:
		try:
			while True:
				linebuf = raw_input("(%s) " % shell.context_name())
				line = shlex.split(linebuf, posix=posix_lex)

				#Catch exception outside the loop so we stop invoking submethods if a parent
				#fails because then the context and results would be unpredictable
				try:
					while len(line) > 0:
						line, finished = shell.invoke(line)
				except APIError as e:
					traceback.print_exc()
				except MoMoException as e:
					print e.format()

				if shell.finished():
					return 0

		#Make sure to catch ^C and ^D so that we can cleanly dispose of subprocess resources if 
		#there are any.
		except EOFError:
			print ""
		except KeyboardInterrupt:
			print ""



#prices.py
#An object encapsulating a list of pricebreaks

from decimal import Decimal

def convert(arg, **kwargs):
	if isinstance(arg, PriceList):
		return arg

	if isinstance(arg, basestring):
		raise ValueError("Converting to price_list from string is not yet supported")

	return PriceList(arg)

def default_formatter(arg, **kwargs):
	return str(arg)

class PriceList:
	def __init__(self, breaks):
		self.breaks = []

		for i in xrange(0, len(breaks)):
			q,val = breaks[i]
			price = Decimal(val)
			self.breaks.append((q, price))

	def unit_price(self, quantity):
		if len(self.breaks) == 0:
			return None

		for i in xrange(0, len(self.breaks)):
			if self.breaks[i][0] > quantity:
				if i == 0:
					return None

				return self.breaks[i-1][1]

		return self.breaks[-1][1]

	def total_price(self, quantity):
		up = self.unit_price(quantity)

		if up is None:
			return None

		return quantity*up

	def __str__(self):
		breaklist = map(lambda x: "%d: %s" % (x[0], x[1]), self.breaks)

		return "\n".join(breaklist)


#ranges.py

def extract_ranges(ih):
	"""
	Given an intelhex16bit object, extract all the disjoint ranges of sequential
	addresses that are defined in the object.
	"""
	valid_addr = map(lambda x:x/2, filter(lambda x: x%2==0, ih.addresses()))
	rstart = valid_addr[0]

	ranges = []

	for i in xrange(1, len(valid_addr)):
		if valid_addr[i] != valid_addr[i-1] + 1:
			rend = valid_addr[i-1]
			ranges.append((rstart, rend))
			rstart = valid_addr[i]

	ranges.append((rstart, valid_addr[-1]))

	return ranges

def build_range(start, end):
	r = (start, end)
	return (r, build_range_func(r))

def build_range_func(range):
		def range_func(x):
			if x >= range[0] and x <= range[1]:
				return True

			return False

		return range_func


from SCons.Script import *
from SCons.Environment import Environment
import sys
import os.path
from utilities import BufferedSpawn
from cfileparser import ParsedCFile

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
from pymomo.utilities.paths import MomoPaths
from pymomo.sim.simulator import Simulator

all_envs = []

def build_module(module_name, chip, postprocess_hex):
	"""
	Configure Scons to build a hex module for the pic24 chip listed in the argument.
	"""

	dirs = chip.build_dirs()
	output_name = '%s.elf' % (chip.output_name(),)
	output_hex = '%s.hex' % (chip.output_name(),)
	postproc_hex = '%s_postprocessed.hex' % (chip.output_name(),)

	VariantDir(dirs['build'], 'src', duplicate=0)

	env = Environment(tools=['xc16_compiler', 'xc16_assembler', 'xc16_linker', 'ldf_compiler'], ENV = os.environ)
	env.AppendENVPath('PATH','../../tools/scripts')
	env['ARCH'] = chip
	env['OUTPUT'] = output_name
	env['CPPPATH'] = chip.includes()

	Export('env')

	SConscript(os.path.join(dirs['build'], 'SConscript'))

	elffile = os.path.join(dirs['build'], output_name)
	hexfile = os.path.join(dirs['build'], output_hex)

	env.Command(hexfile, elffile, 'xc16-bin2hex %s' % elffile)

	if postprocess_hex is not None:
		finalhex = os.path.join(dirs['build'], postproc_hex)

		env.Command(finalhex, hexfile, action=env.Action(postprocess_hex, 'Postprocessing hex file'))
	else:
		finalhex = hexfile

	output = env.InstallAs(os.path.join(dirs['output'], output_hex), finalhex)
	return [output]

def build_library(name, chip):
	"""
	Build the pic24 shared library for the given chip returning an absolute
	path to the product.  
	Parameters are:
	- name: name of the shared library
	- chip: a ChipSettings object for the target chip.
	"""

	dirs = chip.build_dirs()

	libdir = os.path.join(MomoPaths().modules, 'shared', 'pic24')
	builddir = os.path.join(libdir, dirs['build'])
	outdir = os.path.join(libdir, dirs['output'])

	output_name = '%s.a' % (chip.output_name(),)

	VariantDir(builddir, 'src', duplicate=0)

	library_env = Environment(tools=['xc16_compiler', 'xc16_assembler', 'xc16_linker', 'ldf_compiler'], ENV = os.environ)
	library_env.AppendENVPath('PATH','../../tools/scripts')
	library_env['ARCH'] = chip
	library_env['OUTPUT'] = output_name
	library_env['CPPPATH'] = chip.includes()
	
	SConscript(os.path.join(builddir, 'SConscript'), exports='library_env')

	libfile = library_env.InstallAs(os.path.join(outdir, output_name), os.path.join(builddir, output_name))
	return os.path.join(outdir, output_name)

def build_moduletest(test, arch):
	"""
	Given a path to the source files, build a unit test including the unity test harness targeting
	the given architecture.
	"""

	rawlog = '#' + test.get_path('rawlog', arch)
	outlog = '#' + test.get_path('log', arch)
	statusfile = '#' + test.get_path('status', arch)
	elffile = '#' + test.get_path('elf', arch)

	build_dirs = test.build_dirs(arch)
	objdir = build_dirs['objects']

	arch = arch.retarget(add=['test'])

	unit_env = Environment(tools=['xc16_compiler', 'xc16_assembler', 'xc16_linker'], ENV = os.environ)
	tester_env = Environment(tools=['xc16_compiler' ], ENV = os.environ)
	unit_env['ARCH'] = arch
	tester_env['ARCH'] = arch
	unit_env['OUTPUT'] = '%s.elf' % test.name

	objs = []
	for src in test.files + arch.extra_sources():
		name,ext = os.path.splitext(os.path.basename(src))
		target = os.path.join(objdir, name + '.o')

		if src == test.files[0]:
			objs.append(tester_env.xc16_gcc('#' + target, src))
		else:
			objs.append(unit_env.xc16_gcc('#' + target, src))

	#Generate main.c programmatically to run the test
	main_src = '#' + os.path.join(objdir, 'main.c')
	main_target = '#' + os.path.join(objdir, 'main.o')
	unit_env.Command(main_src, test.files[0], action=unit_env.Action(build_moduletest_main, "Creating test runner"))
	objs.append(unit_env.xc16_gcc(main_target, main_src))

	#Link the test, run it and build a status file
	unit_env.xc16_ld(elffile, objs)
	unit_env.Command(outlog, elffile, action=unit_env.Action(r"momo-picunit %s '%s' '%s'" % (arch.property('simulator_model'), elffile[1:], outlog[1:]), "Running unit test")) 
	unit_env.Command(statusfile, outlog, action=unit_env.Action(process_log, 'Processing log file'))

	return statusfile

mainscript = """
int main(void)
{
	U1MODEbits.UARTEN = 1; //Enable the uart
	U1STAbits.UTXEN = 1; //Enable transmission

	UnityBegin("test/TESTNAME");

	if (TEST_PROTECT())
	{
TESTCALLS
	}

	UnityEnd();

	return 0;
}
"""

def build_moduletest_main(target, source, env):
	"""
	Given a module test file, parse it using pycparser, extract all function definitions
	that begin with test_ and then build a main.c file at target containing a test runner
	that calls those functions.
	"""

	name = os.path.basename(str(source[0]))

	srcpath = str(source[0])
	parsed = ParsedCFile(srcpath, env['ARCH'])
	funcs = parsed.defined_functions(criterion=lambda x: x.startswith('test_'))

	testcalls = ""
	testprotos = ""
	for func in funcs:
		testprotos += 'void %s(void);\n' % func
		testcalls += "\t\tRUN_TEST(%s);\n" % func

	with open(str(target[0]), "w")as f:
		f.write('#include "unity.h"\n')
		f.write('#include <xc.h>\n\n')
		f.write(testprotos)

		script = mainscript.replace('TESTNAME', name).replace('TESTCALLS', testcalls)
		f.write(script)

def process_log(target, source, env):
	with open(str(source[0]), "r") as log:
		log.seek(-3, os.SEEK_END)
		status = log.read(2)
	
	with open(str(target[0]), "w") as statfile:
		msg = "FAILED"
		if status == "OK":
			msg = 'PASSED'

		statfile.write(msg)




import unittest
import os.path
import os
from nose.tools import *
from nose.plugins.skip import SkipTest
from pymomo.exceptions import *
from pymomo.pcb import CircuitBoard
from pymomo.pcb.production import ProductionFileGenerator
from distutils.spawn import find_executable
import pymomo
import tempfile
import shutil
import atexit

class TestProductionFileGeneration(unittest.TestCase):
	"""
	Test to make sure that BOM matching and BOM generation work.
	"""

	def _create_tempfile(self, src=None, ext=None):
		"""
		Create a named temporary file that is a copy of the src file
		and register for it to be deleted at program exit.
		"""
		dst = tempfile.NamedTemporaryFile(delete=False)

		if src is not None:
			with open(src, "r+b") as f:
				shutil.copyfileobj(f, dst)

		dst.close()

		name = dst.name
		if ext is not None:
			name = dst.name+"." + ext
			os.rename(dst.name, name)

		atexit.register(os.remove, name)
		return name

	def _create_tempdir(self):
		path = tempfile.mkdtemp()
		atexit.register(shutil.rmtree, path)
		return path

	def setUp(self):
		self.srcbrd = self._create_tempfile(os.path.join(os.path.dirname(__file__), 'eagle', 'controller_complete.brd'), ext='brd')
		cachefile = self._create_tempfile(os.path.join(os.path.dirname(__file__), 'eagle', 'pcb_part_cache.db'))
		
		#Make sure we use our cache file and don't let it expire so we don't delete all of the entries
		pymomo.pcb.partcache.default_cachefile(cachefile)
		pymomo.pcb.partcache.default_noexpire(True)
		self.board = CircuitBoard(self.srcbrd)
		self.board.set_match_engine("CacheOnlyMatcher")

	def tearDown(self):
		pass

	def test_basic(self):
		"""
		Basic tests of ProductionFileGenerator
		"""

		prod = ProductionFileGenerator(self.board)
		prod.save_readme(self._create_tempfile())

	def test_prod(self):
		try:
			eagle = self.board.board._find_eagle()
		except EnvironmentError:
			raise SkipTest("Eagle could not be found")

		print eagle

		out = self._create_tempdir()
		self.board.generate_production(out)



#pv_log.py
#Process the log file created by the pv_module into a csv file

import sys
import csv
import cmdln
import struct
import os.path

class LogFile:
	intervals = {'1s': 0, '2s': 1, '4s': 2,'8s': 3,'16s': 4,'32s': 5,'64s': 6,'128s': 7,'256s': 8}
	int_times = [1, 2 ,4 , 8, 16, 32, 64, 128, 256]
	inv_map = ['1 second', '2 seconds', '4 seconds','8 seconds','16 seconds','32 seconds','64 seconds','128 seconds','256 seconds']

	default_int = 6

	def __init__(self, path):
		self.size = os.path.getsize(path)
		self.file = open(path, "r+b")
		self._read_info()
		self._calculate()

	def _read_info(self):
		self.file.seek(0)
		fmt = "<IIB"

		data = self.file.read(9)

		(magic, valid_sectors, int_num) = struct.unpack(fmt, data)


		if magic == 0x78563412:
			self.valid_file = True
			self.valid_sectors = valid_sectors-1

			if int_num < len(self.inv_map):
				self.interval = self.inv_map[int_num]
				self.intnum = int_num
			else:
				self.interval = "INVALID (value=%d)" % int_num
				self.intnum = self.default_int

		else:
			self.valid_file = False
			self.valid_sectors = -1
			self.interval = "INVALID"
			self.intnum = self.default_int

	def _calculate(self):
		self.num_entries = self.valid_sectors * 512 / 16
		self.remaining = (self.size - 512) / 16 - self.num_entries
		self.time_remaining = self.remaining * self.int_times[self.intnum] / 60 / 60 / 24

	def process(self, v1, v2, v3, i1):
		v1_o = v1/1024. * 20*2.8/2.78
		v2_o = v2/1024. * 20*2.8/2.78
		v3_o = v3/1024. * 25*2.8/2.78
		i1_o = i1/1024. * 2.78 / 0.001 / 100

		return {"v1": v1_o, "v2": v2_o, "v3": v3_o, "i1": i1_o}

	def enum_entries(self, skip, num):
		entry_fmt = "<IIHHHH"
		
		if num < 0:
			num = self.num_entries

		for i in xrange(skip, min(self.num_entries, skip+num)):
			self.file.seek(512 + i*16)
			data = self.file.read(16)
			(mag1, mag2, v1, v2, v3, i1) = struct.unpack(entry_fmt, data)

			if mag1 != 0xDDCCBBAA or mag2 != 0x9955FFEE:
				continue

			yield ((v1, v2, v3, i1), self.process(v1, v2, v3, i1))

	def map_interval(self, interval):
		if interval in self.intervals:
			return self.intervals[interval]

		return self.default_int

	def close(self):
		self.file.close()

	def write_toc(self, valid, interval):
		self.file.seek(0)
		
		fmt = "<IIB"

		if interval is None:
			interval = self.intnum
		elif interval not in self.intervals:
			print "WARNING: Invalid interval specified, using default value"

		int_num = self.map_interval(interval)

		#A valid value < 1 means to keep the current number of valid sectors
		if valid < 1:
			valid = self.valid_sectors+1

		header = struct.pack(fmt, 0x78563412, max(valid, 1), int_num)
		self.file.write(header)
		self.file.flush()

class PVCommands(cmdln.Cmdln):
	name = 'pv_log'

	def do_info(self, subcmd, opts, log_file):
		"""${cmd_name}: Print info about the log file specified

		Detailed information is given about the number of log entries,
		the log file size and other interesting information.
		${cmd_usage}
		${cmd_option_list}
		LOG_FILE should be a path to a log file (log.bin) produced by the
		MoMo solar logging module.
		"""

		try:
			log = LogFile(log_file)
		except IOError:
			print "Error: could not open log file: %s" % log_file
			return 1

		print "Log File Information"
		print "File Valid: %s" % log.valid_file
		print "Number of Sectors Used: %d" % log.valid_sectors
		print "Number of Datapoints Logged: %d" % log.num_entries 
		print "Logging Interval: %s (numerical value %d) " % (log.interval, log.intnum)
		print "Space Remaining: %d entries" % log.remaining
		print "Time Remaining: %d days" % log.time_remaining
		return 0

	@cmdln.option('-s', '--skip', action='store', type=int, default=0, help='skip the first SKIP entries')
	@cmdln.option('-n', '--num', action='store', type=int, default=-1, help='print only NUM entries')
	def do_print(self, subcmd, opts, log_file):
		"""${cmd_name}: Print all or a subset of the log entries to stdout

		${cmd_usage}
		${cmd_option_list}
		LOG_FILE should be a path to a log file (log.bin) produced by the
		MoMo solar logging module.
		"""

		try:
			log = LogFile(log_file)
		except IOError:
			print "Error: could not open log file: %s" % log_file
			return 1

		for raw, proc in log.enum_entries(opts.skip, opts.num):
			print "V1: %.2fV || V2: %.2fV || V3: %.2fV || Current: %.2fA" % (proc['v1'], proc['v2'], proc['v3'], proc['i1'])

		return 0

	@cmdln.option('-o', '--out', action='store', default='log.csv', help='output CSV file to be generated')
	@cmdln.option('-s', '--skip', action='store', type=int, default=0, help='skip the first SKIP entries')
	@cmdln.option('-n', '--num', action='store', type=int, default=-1, help='print only NUM entries')
	def do_convert(self, subcmd, opts, log_file):
		"""${cmd_name}: Convert a binary log file (log.bin) to a CSV spreadsheet

		${cmd_usage}
		${cmd_option_list}
		LOG_FILE should be a path to a log file (log.bin) produced by the
		MoMo solar logging module.
		"""

		try:
			log = LogFile(log_file)
		except IOError:
			print "Error: could not open log file: %s" % log_file
			return 1

		try:
			out = open(opts.out, "w")
			writer = csv.writer(out, lineterminator='\n')
		except IOError:
			log.close()
			print "Error: could not create output file: %s" % opts.out
			return 1

		headers = ['Voltage 1 (V)', 'Voltage 2 (V)', 'Voltage 3 (V)', 'Current (A)', 'Raw V1', 'Raw V2', 'Raw V3', 'Raw Current']
		writer.writerow(['WellDone PV Logging Module v1'])
		writer.writerow(['Logging Interval: %s' % log.interval])
		writer.writerow(headers)
		for raw, proc in log.enum_entries(opts.skip, opts.num):
			row = [proc['v1'], proc['v2'], proc['v3'], proc['i1'], raw[0], raw[1], raw[2], raw[3]]
			writer.writerow(row)

		out.close()

		return 0

	@cmdln.option('-s', '--size', action='store', type=int, default=1024*1024*128, help='file size (default is 128MB)')
	@cmdln.option('-i', '--interval', type='choice', choices=['1s', '2s', '4s','8s','16s','32s','64s','128s','256s'], default='64s', help='Logging interval (default: 64s)')
	def do_create(self, subcmd, opts, directory):
		"""${cmd_name}: Create a new log file in the specified directory

		${cmd_usage}
		${cmd_option_list}
		DIRECTORY should be a path to the root directory of the SD card that 
		will be used for logging.  The log file must be completely preallocated,
		meaning that the size you specify in this command will limit how large of
		a log the data logger can produce.  The logger will log data at the interval
		specified with the -i option.  

		Possible logging intervals are 1s, 4s, 8s, 16s, 32s, 64s (default), 128s and 256s.
		"""

		block = 4096
		numblocks = opts.size / block
		remainder = opts.size % block

		log_file = os.path.join(directory, 'log.bin')

		blockbuff = "\0" * block

		with open(log_file, "w") as f:
			for i in xrange(0, numblocks):
				f.write(blockbuff)

			remblock = "\0" * remainder
			f.write(remblock)

		try:
			log = LogFile(log_file)
		except IOError:
			print "Error: could not open log file: %s" % log_file
			return 1

		log.write_toc(1, opts.interval)
		log.close()

		return 0

	@cmdln.option('-t', '--truncate', action='store_true', help='erase all logged data')
	@cmdln.option('-i', '--interval', type='choice', choices=['1s','2s','4s','8s','16s','32s','64s','128s','256s'], default=None, help='Logging interval (default: 64s)')
	def do_adjust(self, subcmd, opts, log_file):
		"""${cmd_name}: Create a new log file in the specified directory

		${cmd_usage}
		${cmd_option_list}
		Adjust the logging settings specified by this log.bin file.  You can update
		the logging interval and/or erase all of the logged data in the file
		LOG_FILE should be a path to a log file (log.bin) produced by the
		MoMo solar logging module.

		Possible logging intervals are 1s, 4s, 8s, 16s, 32s, 64s (default), 128s and 256s.
		"""

		valid = -1

		if opts.truncate:
			valid = 1

		try:
			log = LogFile(log_file)
		except IOError:
			print "Error: could not open log file: %s" % log_file
			return 1

		log.write_toc(valid, opts.interval)
		log.close()

		return 0

if __name__ == "__main__":
	pv = PVCommands()
	sys.exit(pv.main())


from logstatement import LogStatement

class LogCaller (LogStatement):
	def __init__(self, statement, log):
		if len(statement['data']) != 2:
			raise ValueError('Invalid hex logging statement, data length != 2 (len=%d)' % len(statement['data']))

		addr_low = statement['data'][0]['value']
		addr_high = statement['data'][1]['value']

		self.address = addr_high*256 + addr_low
		log.current_address = self.address-1 		#We log the return address, which is the next instruction to be executed

	def keep(self):
		return False

	def format(self):
		#This just mutates Log object state, it prints nothing
		return None


import sys
import os.path
from pymomo.utilities import build

class MIBHandler:
	"""
	A class representing a MIB callback routine suitable for processing those found in mib blocks
	pulled from hex files or for creating those blocks based on descriptions of the handlers.
	"""
	BufferSizeMask = 0b11111
	BufferFlag = 1<<7

	def __init__(self, spec, symbol=None, address=None):
		self.symbol = symbol
		self.address = address
		self.spec = spec

		self._validate()

	@classmethod
	def Create(cls, symbol, ints=0, has_buffer=False):
		if ints > 4:
			raise ValueError("Invalid MIBHandler Spec: too many int parameters. (value = %d"  % ints)
		if ints < 0:
			raise ValueError("Invalid MIBHandler Spec: negative number of int parameters. (value = %d)" % ints)

		spec = 0

		if has_buffer:
			spec = 1 << 7

		spec |= (ints&0b11) << 5

		return MIBHandler(spec, symbol)

	def num_ints(self):
		return (self.spec >> 5) & 0b11

	def has_buffer(self):
		return self.spec & MIBHandler.BufferFlag

	def _validate(self):
		if self.symbol is None and self.address is None:
			raise ValueError("You must pass either a symbol or an address to MIBHandler")

	def _format_spec(self):
		ints = self.num_ints()
		buffer = self.has_buffer()

		if buffer and (ints==0):
			return "buffer only"
		elif buffer:
			return "%d integers and a buffer" % ints
		elif (ints==0) and not buffer:
			return "no parameters"

		return "%d integers" % ints

	def __str__(self):
		name = "<unnamed> "
		addr = "<no address>"

		if self.symbol is not None:
			name = self.symbol + " "

		if self.address is not None:
			addr = "0x%X" % self.address

		return "%s@ %s (%s)" % (name, addr, self._format_spec())


#!/usr/bin/env python

import sys
import os.path
import os
import intelhex

import cmdln
from colorama import Fore, Style
from pymomo.hex8.instruction import Instruction
from pymomo.hex8 import merge
from pymomo.hex8.ranges import *

class HexTool(cmdln.Cmdln):
	name = 'hextool'

	@cmdln.option('-o', '--output', action='store',  help='output hex file')
	@cmdln.option('-a', '--addr', action='store', help="the address to patch")
	@cmdln.option('-v', '--verify', action='store', default=None, help="verify the previous contents before patching")
	@cmdln.option('-r', '--replace', action='store')
	@cmdln.option("-p", '--pic24', action="store_true", default=False, help='Is the hex file for a 16 bit PIC' )
	def do_patch(self, subcmd, opts, hexfile):
		"""${cmd_name}: patch a 16 bit pic12 or pic16 hex file

		Replace the instruction at address --addr with the instruction
		given by --replace.  Save the patched file in --output and optionally
		verify that the previous contents at --addr were --verify.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['addr', 'output', 'replace'])
		verify = None
		addr = self.parse_num(opts.addr)

		#Do the patching
		ih = self.load_ih(hexfile, pic24=opts.pic24)

		if opts.pic24 == False:
			repl = self.parse_instr(opts.replace)
			if opts.verify is not None:
				verify = self.parse_instr(opts.verify)

			if verify is not None:
				op = verify.encode()
				if ih[addr] != op:
					self.error("Verification failed, old address contents: 0x%X, expected 0x%X for '%s'" % (ih[addr], op, str(verify.op)))

			print "Replacing Instruction at 0x%X with %s" % (addr, repl.op)
			ih[addr] = repl.encode()
		else:
			repl = self.parse_num(opts.replace)
			print "Replacing 0x%X at address 0x%X with 0x%X" % (ih[addr*2], addr, repl)
			ih[addr*2] = repl

		ih.write_hex_file(opts.output)

	@cmdln.option('-o', '--output', action='store',  help='output hex file')
	@cmdln.option('-a', '--addr', action='store', help="the address to start inserting")
	@cmdln.option('-s', '--skip', action='append', help="skip address range inclusive: start,end")
	@cmdln.option('-m', '--maxsize', action='store', default='0', help="the maximum size to insert")
	def do_insert(self, subcmd, opts, stub_file, payload_file):
		"""${cmd_name}: merge payload into stub at a given offset

		Write the hex file specified by payload sequentially into the
		address space of stub starting at addr and optionally skipping
		address ranges specified with --skip.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['output', 'addr'])
		stub = self.load_ih(stub_file)
		payload = self.load_ih(payload_file)

		#Write payload into stub starting at write_addr
		#and skipping over the ranges listed in skip_ranges
		skip_ranges = self.parse_ranges(opts.skip)
		write_addr = self.parse_num(opts.addr)
		max_size = self.parse_num(opts.maxsize)

		print "Inserting hex file at address: 0x%X" % write_addr

		merge.insert_hex(stub, payload, write_addr, skip_ranges, max_size)
		stub.write_hex_file(opts.output)

	@cmdln.option('-o', '--output', action='store',  help='output hex file')
	@cmdln.option('-a', '--addr', action='store', help="the address to start extracting")
	@cmdln.option('-s', '--skip', action='append', help="skip address range inclusive: start,end")
	@cmdln.option('-m', '--maxsize', action='store', default='0', help="the maximum size to extract")
	def do_extract(self, subcmd, opts, hexfile):
		"""${cmd_name}: merge payload into stub at a given offset

		Write the hex file specified by payload sequentially into the
		address space of stub starting at addr and optionally skipping
		address ranges specified with --skip.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		self.assert_args(opts, ['output', 'addr'])
		stub = self.load_ih(hexfile)

		#Write payload into stub starting at write_addr
		#and skipping over the ranges listed in skip_ranges
		skip_ranges = self.parse_ranges(opts.skip)
		write_addr = self.parse_num(opts.addr)
		max_size = self.parse_num(opts.maxsize)

		print "Extracting hex file at address: 0x%X" % write_addr

		out = merge.extract_hex(stub, write_addr, skip_ranges, max_size)
		out.write_hex_file(opts.output)

	def do_merge(self, subcmd, opts, file1, file2, file3):
		"""${cmd_name}: merge two pic24 hex files together

		Combine file1 and file2, where the contents of file2 overlap the contents of file1,
		file2 will overwrite file1.  The result is saved into file3
		
		${cmd_usage}
		${cmd_option_list}
		"""

		ih1 = intelhex.IntelHex(file1)
		ih2 = intelhex.IntelHex(file2)

		ih1.merge(ih2, overlap='replace')
		ih1.write_hex_file(file3)

	@cmdln.option("-p", '--pic24', action="store_true", default=False, help='Is the hex file for a 16 bit PIC' )
	def do_compare(self, subcmd, opts, file1, file2):
		"""${cmd_name}: compare file1 and file2 

		Compare the contents of file1 and file2.  The comparison is 
		done 2 ways.  First all of the addresses defined in file1 
		are compared with fil2, then all of the addresses defined in 
		file2 are compared with file1.  This lets you find out if one
		is a subset of the other.
		
		${cmd_usage}
		${cmd_option_list}
		"""

		ih1 = self.load_ih(file1, opts.pic24)
		ih2 = self.load_ih(file2, opts.pic24)

		ranges1 = extract_ranges(ih1)
		ranges2 = extract_ranges(ih2)

		print "\nFile 1 defined address ranges:"
		for r in ranges1:
			print "[0x%X, 0x%X]" % (r[0], r[1])

		print "\nFile 2 defined address ranges:"
		for r in ranges2:
			print "[0x%X, 0x%X]" % (r[0], r[1])

		mismatch = False
		print "\nComparing ranges defined in file1"
		for start,end in ranges1:
			for i in xrange(start, end+1):
				if ih1[i] != ih2[i]:
					mismatch = True
					print "Mismatch at 0x%X: file1=0x%X, file2=0x%X" % (i, ih1[i], ih2[i])

		if not mismatch:
			print "Ranges matched."

		mismatch = False
		print "\nComparing ranges defined in file2"
		for start,end in ranges2:
			for i in xrange(start, end+1):
				if ih1[i] != ih2[i]:
					mismatch = True
					print "Mismatch at 0x%X: file1=0x%X, file2=0x%X" % (i, ih1[i], ih2[i])

		if not mismatch:
			print "Ranges matched."

	def do_ranges(self, subcmd, opts, hexfile):
		"""${cmd_name}: print the address ranges defined in hex file
		
		${cmd_usage}
		${cmd_option_list}
		"""

		ih = self.load_ih(hexfile)

		valid_addr = map(lambda x:x/2, filter(lambda x: x%2==0, ih.addresses()))
		rstart = valid_addr[0]

		print "\nListing Address Ranges"
		for i in xrange(1, len(valid_addr)):
			if valid_addr[i] != valid_addr[i-1] + 1:
				rend = valid_addr[i-1]
				print "Range: 0x%X-0x%X" % (rstart, rend)

				rstart = valid_addr[i]

		print "Range: 0x%X-0x%X" % (rstart, valid_addr[-1])

	def load_ih(self, hexfile, pic24=False):
		try:
			if not pic24:
				ih = intelhex.IntelHex16bit(hexfile)
			else:
				ih = intelhex.IntelHex(hexfile)
		except IOError as e:
			self.error(str(e))

		ih.padding = 0x3FFF

		return ih

	def parse_instr(self, instr):
		try:
			ins = Instruction(instr)
			return ins
		except ValueError as e:
			self.error(str(e))

	def parse_ranges(self, ranges_str):
		ranges = map(lambda x:x.split('-'), ranges_str)
		ranges = map(lambda x: (int(x[0].lstrip().rstrip(),0), int(x[1].lstrip().rstrip(),0)), ranges)
		ranges = map(lambda x: (x, self._build_range_func(x)), ranges)
		return ranges

	def _build_range_func(self, range):
		def range_func(x):
			if x >= range[0] and x <= range[1]:
				return True

			return False

		return range_func

	def parse_num(self, num):
		return int(num, 0)

	def assert_args(self, opts, args):
		for arg in args:
			if not hasattr(opts, arg) or getattr(opts, arg) is None:
				self.error("You must specify an argument for %s" % arg)

	def error(self, text):
		print Fore.RED + "Error Occurred: " + Style.RESET_ALL + text
		sys.exit(1)

def main():
	hextool = HexTool()
	return hextool.main()


from externalsim import ExternalSimulator
from pymomo.utilities.config import ConfigFile
import subprocess
from pymomo.utilities.typedargs.annotate import *
from pymomo.exceptions import *
from pymomo.utilities.asyncio import AsyncLineBuffer
from collections import namedtuple

messages = {
	'CORE-W0008: Software Reset Instruction called at PC=': '_parse_reset'
}

#Types of sim30 messages
SoftwareResetMessage = namedtuple('SoftwareResetMessage', ['pc'])

class SIM30 (ExternalSimulator):
	"""
	An ExternalSimulator object encapsulating the sim30 pic24 simulator from microchip
	"""

	prompt = '\rdsPIC30> '
	known_params = set(['model'])

	def _sim_path(self):
		return ['sim30']

	def _initialize(self):
		self.model = 'pic24super'
		self._init_sim()

	def _init_sim(self):
		self._set_mode(self.model)
		self.first_run = True

	def _process_message(self, msg, cmd=None):
		"""
		Given the textual output of a sim30 response to a command, parse it and
		see if it indicates an error.  Convert the result to a standard format
		for further inspection as well.
		"""

		msg = msg.rstrip()
		lines = [x for x in msg.split('\n') if len(x) > 0]

		parsed = []

		for line in lines:
			matches = [messages[x] for x in messages.iterkeys() if line.startswith(x)]
			if len(matches) == 0:
				raise APIError("Unknown message from sim30: %s" % line)
			elif len(matches) > 1:
				raise APIError("Multiple matches for message from sim30: %s" % line)

			parser = getattr(self, matches[0])
			parsed.append(parser(line))
		
		return parsed

	def _set_mode(self, mode):
		result = self._command('LD %s' % mode, noparse=True)

	### BEGIN SIMULATION COMMANDS ###
	def load_program(self, program, type):
		#FIXME, look for spaces in program path and raise an
		#error.  sim30 cannot handle spaces in program paths.
		cmd = 'LP'

		if type == 'coff' or type== 'elf':
			cmd = 'LC'

		result = self._command(cmd, program)
		if len(result) != 0:
			raise InternalError(result)

		#We need to 
		self.first_run = True

	def set_param(self, name, value):
		if name in self.known_params:
			setattr(self, name, value)

		if name == 'model':
			self._init_sim()

	def set_log(self, file):
		self._command('io nul %s' % file)

	def execute(self):
		#sim30 requires the processor to be reset before executing
		#code the first time or it dies for an unknown reason.
		if self.first_run:
			self._command('RP')
			self.first_run = False

		#Stop execution on each warning so that we can regain control after a 
		#reset instruction.
		self._command('HW on')
		self._command('E', sync=False)
		self.waiting = True


	#Utility routines for parsing specific messages from sim30
	def _parse_reset(self, line):
		skip = len('CORE-W0008: Software Reset Instruction called at PC=')
		pcstr = line[skip:]
		pc = int(pcstr, 16)

		msg = SoftwareResetMessage(pc)
		return msg
